/*
* Enhydra Java Application Server Project
*
* The contents of this file are subject to the Enhydra Public License
* Version 1.1 (the "License"); you may not use this file except in
* compliance with the License. You may obtain a copy of the License on
* the Enhydra web site ( http://www.enhydra.org/ ).
*
* Software distributed under the License is distributed on an "AS IS"
* basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
* the License for the specific terms governing rights and limitations
* under the license.
*
* The Initial Developer of the Enhydra Application Server is Lutris
* Technologies, Inc. The Enhydra Application Server and portions created
* by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
* All Rights Reserved.
*
* Contributor(s):
* Paul Mahar
*
 */
package org.enhydra.kelp.common.node;

// ToolBox
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.archive.Descriptor;
import org.enhydra.tool.archive.JarPlan;
import org.enhydra.tool.archive.WarPlan;
import org.enhydra.tool.archive.ArchiveException;
import org.enhydra.tool.archive.PropertyReader;
import org.enhydra.tool.archive.PropertyWriter;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.configure.ConfigTool;

// AddinCore
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.Backward;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.PropUtil;
import org.enhydra.kelp.common.map.MapEntry;
import org.enhydra.kelp.common.map.MapUtil;

// JDK
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Vector;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Properties;
import org.enhydra.dods.generator.DODSGenerator;


//
abstract public class OtterProject implements OtterNode {
  public final static int MAP_SCOPE_ALL = 0;
  public final static int MAP_SCOPE_SELECTED = 1;
  public final static int MAP_SCOPE_NONE = 2;

  //
  public final static int TYPE_UNKNOWN = -1;
//Dusan 05.02.2003.
//    public final static int TYPE_WEBAPP = 0;
  public final static int TYPE_WEBAPP = 0;
//    public final static int TYPE_SERVICE = 1;
//    public final static int TYPE_EN3APP = 2;
  public final static int TYPE_EN3APP = 1;

  private DODSGenerator generator = null;
  private boolean documentationHtml;
  private boolean documentationPdf;
  private boolean documentationXmi;
  private boolean documentationPtl;

  //
  abstract public String getCodeGenDefaultDestination();

  /**
   * Find and return first .policy node found in project.
   */
  abstract public OtterTextFileNode getFirstPolicy();

  /**
   * Get working directory for project run.
   */
  abstract public String getWorkingPath();

  /**
   * Set working directory for project run.
   */
  abstract public void setWorkingPath(String path);

  /**
   * Return all web.xml nodes.
   */
  abstract public OtterTextFileNode[] getAllDeploymentDescs();

  /**
   * Return all nodes that ends with an extension in
   * the provided extension list.
   *
   * @param extensions
   */
  abstract public OtterFileNode[] findFileNodesByType(String[] extensions);

  /**
   * Get all templates contained in the input directory.
   */
  abstract public OtterFileNode[] getAllInput();

  /**
   * Get all document nodes in the project.
   */
  abstract public OtterDocumentNode[] getAllDocuments();

  /**
   * Return the IDE implementation of the project node.
   */
  abstract public Object getNativeProject();
  abstract public String getRootPath();

  /**
   * Does IDE allow for integrated build.
   */
  abstract public boolean isOpenBuild();

  /**
   * Method declaration
   *
   * @return
   */
  abstract public String getClassOutputPath();
  abstract public void setClassOutputPath(String p);

  //
  abstract public boolean isDefaultProject();

  /**
   * Method declaration
   *
   * @return
   */
  abstract public String getSourcePath();

  /**
   * Method declaration
   *
   * @return
   */
  abstract public String[] getSourcePathArray();

  /**
   * Method declaration
   *
   * @param pathAsArray
   */
  abstract public void setSourcePathArray(String[] pathAsArray);

  /**
   * Method declaration
   *
   *
   * @return
   */
  abstract public String getClassPath();

  /**
   * Method declaration
   *
   * @param property
   *
   * @return
   */
  abstract public String getProperty(String property);

  /**
   * Method declaration
   *
   * @return
   */
  abstract public OtterNodeFactory getNodeFactory();

  /**
   * Method declaration
   *
   */
  abstract public void configureRunClass();

  /**
   * Method declaration
   *
   * @param property
   * @param value
   */
  abstract public void setProperty(String property, String value);

  /**
   * Method declaration
   *
   * @param property
   * @param value
   */
  abstract public void setProperty(String property, int value);

  //
  //
  public String getProjectRootName() {
    File   root = new File(getRootPath());
    String projectName = root.getName();

    return projectName;
  }

  public String[] getDocTypes() {
    String[] exts = new String[0];
    String   extList = null;

    exts = ToolBoxInfo.getSupportedDocTypes();    // defaults
    extList = getProperty(XMLC_DOC_TYPES);
    exts = PropUtil.listToArray(extList, exts);
    return exts;
  }

  public void setDocTypes(String[] exts) {
    String extList = null;

    extList = PropUtil.arrayToList(exts);
    setProperty(XMLC_DOC_TYPES, extList);
  }

  public String[] getNonDocumentContentTypes() {
    String[] exts = PropertyKeys.DEFAULT_CONTENT;
    String   extTypes = null;

    extTypes = getProperty(DEPLOY_CONTENT_TYPES);
    exts = PropUtil.listToArray(extTypes, exts);
    return exts;
  }

  public String[] getContentTypes() {
    ArrayList list = null;
    String[]  docs = new String[0];
    String[]  exts = getNonDocumentContentTypes();

    // add doc types
    list = new ArrayList(Arrays.asList(exts));
    docs = getDocTypes();
    for (int i = 0; i < docs.length; i++) {
      if (!list.contains(docs[i])) {
        list.add(docs[i]);
      }
    }
//Dusan
/*        if (getDeployType() == OtterProject.TYPE_SERVICE) {
            if (!list.contains("xml")) {
                list.add("xml");
            }
        } */
    list.trimToSize();
            exts = new String[list.size()];
            exts = (String[]) list.toArray(exts);
            return exts;
  }

  public void setContentTypes(String[] exts) {
    ArrayList list = null;
    String[]  docs = new String[0];
    String    extTypes = null;

    // remove doc types
    list = new ArrayList(Arrays.asList(exts));
    docs = getDocTypes();
    for (int i = 0; i < docs.length; i++) {
      if (list.contains(docs[i])) {
        list.remove(docs[i]);
      }
    }
    list.trimToSize();
    exts = new String[list.size()];
    exts = (String[]) list.toArray(exts);
    extTypes = PropUtil.arrayToList(exts);
    setProperty(DEPLOY_CONTENT_TYPES, extTypes);
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public OtterXMLCNode[] getAllXMLCNodes() {
    OtterDocumentNode[] docNodes = new OtterDocumentNode[0];
    OtterXMLCNode[]     xmlcNodes = new OtterXMLCNode[0];

    docNodes = getAllDocuments();
    xmlcNodes = new OtterXMLCNode[docNodes.length];
    for (int i = 0; i < docNodes.length; i++) {
      xmlcNodes[i] = new OtterXMLCNode(docNodes[i]);
    }
    return xmlcNodes;
  }

  /**
   * Method declaration
   *
   * @return
   */
  public String getXMLCOptionFilePath() {
    String filename = new String();
    File   file = PathUtil.getFileRelativeToProject(this,
        PropertyKeys.NAME_XMLC_OPTION_FILEPATH);

    if ((file != null) && file.exists() && (!file.isDirectory())) {
      filename = file.getAbsolutePath();
    }
    return filename;
  }

  /**
   * Method declaration
   *
   * @param filename
   */
  public void setXMLCOptionFilePath(String filename) {
    PathUtil.putFileRelativeToProject(this,
                                      PropertyKeys.NAME_XMLC_OPTION_FILEPATH,
                                      filename);
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public String getXMLCParameters() {
    String params = getProperty(PropertyKeys.NAME_XMLC_PARAMETERS);

    params = PropUtil.removeQuotes(params);
    return params;
  }

  /**
   * Method declaration
   *
   * @param p
   */
  public void setXMLCParameters(String p) {
    setProperty(PropertyKeys.NAME_XMLC_PARAMETERS, p);
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public boolean isPrintDOM() {
    return readBooleanProperty(XMLC_PRINT_DOM, false);
  }

  /**
   * Method declaration
   *
   * @param b
   */
  public void setPrintDOM(boolean b) {
    writeBooleanProperty(XMLC_PRINT_DOM, b);
  }

  /**
   * Method declaration
   *
   * @return
   */
  public boolean isXMLCBuild() {
    return readBooleanProperty(XMLC_BUILD, false);
  }

  /** Dusan
   * Method declaration
   *
   * @return
   */
  public boolean isDodsBuild() {
    return readBooleanProperty(DODS_BUILD, false);
  }

  /**
   * Method declaration
   *
   *
   * @param b
   */
  public void setXMLCBuild(boolean b) {
    writeBooleanProperty(XMLC_BUILD, b);
  }

  /**
   * Method declaration
   *
   * @return
   */
  public int getDeployType() {
    String in = getProperty(DEPLOY_TYPE);
    int    type = TYPE_UNKNOWN;

    type = PropUtil.stringToInt(in, TYPE_UNKNOWN);
    if (type == TYPE_UNKNOWN) {
      if (Backward.createIsWebApp(getSourcePathArray())) {
        type = TYPE_WEBAPP;
      }
    }
    return type;
  }

  /**
   * Method declaration
   *
   * @return
   */
  public int getDodsType() {
    String in = getProperty(DODS_TYPE);
    int    type = TYPE_UNKNOWN;

    type = PropUtil.stringToInt(in, TYPE_UNKNOWN);
    if (type == TYPE_UNKNOWN) {
      if (Backward.createIsWebApp(getSourcePathArray())) {
        type = TYPE_WEBAPP;
      }
    }
    return type;
  }

  /**
   * Method declaration
   *
   * @param b
   */
  public void setDeployType(int type) {
    setProperty(DEPLOY_TYPE, type);
  }

  /**
   * Method declaration
   *
   * @param b
   */
  public void setDodsType(int type) {
    setProperty(DODS_TYPE, type);
  }

  public String getDeployResourcePath() {
    String out = new String();

    out = getProperty(DEPLOY_PATH_RESOURCE);
    if ((out == null) || (out.trim().length() == 0)) {
      out = getSourcePathArray()[0];
    } else {
      out = PathUtil.expandPathRelativeToProject(this, out);
    }
    out = PathHandle.createPathString(out);
    return out;
  }

  public void setDeployResourcePath(String path) {
    path = PathUtil.compressPathRelativeToProject(this, path);
    setProperty(DEPLOY_PATH_RESOURCE, path.replace('\\', '/'));
  }

  public String getDeployRootPath() {
    String out = new String();

    out = getProperty(DEPLOY_PATH_ROOT);
    if ((out == null) || (out.trim().length() == 0)) {
      out = getWorkingPath() + File.separator + Constants.DIR_OUTPUT;
    } else {
      out = PathUtil.expandPathRelativeToProject(this, out);
    }
    return PathHandle.createPathString(out);
  }

/*    public String getDodsRootPath() {
        String out = new String();

        out = getProperty(DODS_PATH_ROOT);
        if ((out == null) || (out.trim().length() == 0)) {
            out = getWorkingPath() + File.separator + Constants.DIR_OUTPUT;
        } else {
            out = PathUtil.expandPathRelativeToProject(this, out);
        }
        return PathHandle.createPathString(out);
    } */

  public void setDeployRootPath(String path) {
    path = PathUtil.compressPathRelativeToProject(this, path);
    setProperty(DEPLOY_PATH_ROOT, path.replace('\\', '/'));
  }

/*    public void setDodsRootPath(String path) {
        path = PathUtil.compressPathRelativeToProject(this, path);
        setProperty(DODS_PATH_ROOT, path.replace('\\', '/'));
    }
*/
  public boolean isDeployInput() {
    boolean def = false;
    int type = getDeployType();

    if (type == OtterProject.TYPE_EN3APP ||
        type == OtterProject.TYPE_WEBAPP) {
      def = true;
    }
    return readBooleanProperty(DEPLOY_INPUT_ENABLE, def);
  }

  public void setDeployInput(boolean b) {
    writeBooleanProperty(DEPLOY_INPUT_ENABLE, b);
  }

  public boolean isDeployContent() {
    boolean def = false;
    int type = getDeployType();

    if (type == OtterProject.TYPE_EN3APP ||
        type == OtterProject.TYPE_WEBAPP
//Dusan
//            type == OtterProject.TYPE_SERVICE) {
        ) {
      def = true;
    }

    return readBooleanProperty(DEPLOY_CONTENT_ENABLE, def);
  }

  public void setDeployContent(boolean b) {
    writeBooleanProperty(DEPLOY_CONTENT_ENABLE, b);
  }

  public boolean isDeployRun() {
    boolean def = true;

    if (getDeployType() == OtterProject.TYPE_UNKNOWN){
      def = false;
    }

    return readBooleanProperty(DEPLOY_RUN_ENABLE, def);
  }

  public void setDeployRun(boolean b) {
    writeBooleanProperty(DEPLOY_RUN_ENABLE, b);
  }


  public String getDeployInputPath() {
    String out = new String();

    out = getProperty(DEPLOY_PATH_INPUT);
    if ((out == null) || (out.trim().length() == 0)) {
      out = getWorkingPath() + File.separator + Constants.DIR_INPUT;
      out = Backward.createDefaultInputPath(out, getSourcePathArray());
    } else {
      out = PathUtil.expandPathRelativeToProject(this, out);
    }
    out = PathHandle.createPathString(out);
    return out;
  }

  public void setDeployInputPath(String path) {
    path = PathUtil.compressPathRelativeToProject(this, path);
    path = path.replace('\\', '/');
    setProperty(DEPLOY_PATH_INPUT, path);
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public boolean isPrintParseInfo() {
    return readBooleanProperty(XMLC_PRINT_PARSE, false);
  }

  /**
   * Method declaration
   *
   *
   * @param b
   */
  public void setPrintParseInfo(boolean b) {
    writeBooleanProperty(XMLC_PRINT_PARSE, b);
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public boolean isPrintAccessorInfo() {
    return readBooleanProperty(XMLC_PRINT_ACCESSOR, false);
  }

  /**
   * Method declaration
   *
   *
   * @param b
   */
  public void setPrintAccessorInfo(boolean b) {
    writeBooleanProperty(XMLC_PRINT_ACCESSOR, b);
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public boolean isPrintDocumentInfo() {
    return readBooleanProperty(XMLC_PRINT_DOC, false);
  }

  /**
   * Method declaration
   *
   *
   * @param b
   */
  public void setPrintDocumentInfo(boolean b) {
    writeBooleanProperty(XMLC_PRINT_DOC, b);
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public boolean isVerbose() {
    return readBooleanProperty(XMLC_VERBOSE, false);
  }

  /**
   * Method declaration
   *
   *
   * @param b
   */
  public void setVerbose(boolean b) {
    writeBooleanProperty(XMLC_VERBOSE, b);
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public int getVerifiedMapScope() {
    int scope = MAP_SCOPE_ALL;

    scope = getMapScope();

    // if selected, verify
    if (scope == MAP_SCOPE_SELECTED) {
      OtterXMLCNode[] nodes = new OtterXMLCNode[0];
      int             mapCount = 0;
      int             type = OtterXMLCNode.CLASS_NAME_MAPPED;

      nodes = getAllXMLCNodes();
      if (nodes.length > 0) {
        for (int i = 0; i < nodes.length; i++) {
          type = nodes[i].getClassNameType();
          if (type == OtterXMLCNode.CLASS_NAME_MAPPED) {
            mapCount++;
          }
        }
        if (mapCount == 0) {
          scope = MAP_SCOPE_NONE;
          setMapScope(MAP_SCOPE_NONE);
        } else if (mapCount == nodes.length) {
          scope = MAP_SCOPE_ALL;
          setMapScope(MAP_SCOPE_ALL);
        }
      }
      nodes = new OtterXMLCNode[0];
    }
    return scope;
  }

  public int getMapScope() {
    String strScope;
    int    intScope;

    strScope = getProperty(XMLC_MAP_SCOPE);
    intScope = PropUtil.stringToInt(strScope, MAP_SCOPE_ALL);
    return intScope;
  }

  /**
   * Method declaration
   *
   *
   * @param b
   */
  public void setMapScope(int i) {
    setProperty(XMLC_MAP_SCOPE, i);
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public String getOutputFilename() {
    String out = null;

    out = getProperty(XMLC_OUTPUT_FILENAME);
    return out;
  }

  /**
   * Method declaration
   *
   *
   * @param n
   */
  public void setOutputFilename(String n) {
    setProperty(XMLC_OUTPUT_FILENAME, n);
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public String[][] getPackageMap() {
    String[][] map = PropUtil.getArrayProperty(this,
        XMLC_MAP_PACKAGE_LENGTH,
        XMLC_MAP_PACKAGE_FROM,
        XMLC_MAP_PACKAGE_TO);
    File       from;

    for (int i = 0; i < map.length; i++) {
      map[i][0] = PathUtil.expandPathRelativeToProject(this, map[i][0]);
    }
    map = MapUtil.optimize(map);
    return map;
  }

  /**
   * Method declaration
   *
   * @param map
   */
  public void setPackageMap(String[][] map) {
    map = MapUtil.optimize(map);
    for (int i = 0; i < map.length; i++) {
      map[i][0] = PathUtil.compressPathRelativeToProject(this,
      map[i][0]);
    }
    PropUtil.putArrayProperty(this, XMLC_MAP_PACKAGE_LENGTH,
                              XMLC_MAP_PACKAGE_FROM, XMLC_MAP_PACKAGE_TO,
                              map);
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public String[][] getReplacementTable() {
    String[][] table = PropUtil.getArrayProperty(this,
        DEPLOY_TABLE_LENGTH, DEPLOY_TABLE_FIND, DEPLOY_TABLE_REPLACE);

    // replace relative paths in replaceWith column
    if (table.length == 0) {
      try {
        table = ConfigTool.createReplacementStringArray(getRootPath(),
            getDeployRootPath(), ToolBoxInfo.getJavaPath());
      } catch (ToolException e) {
        table = new String[0][0];
        e.printStackTrace();
      }
      table =
          Backward.createReplacementTable(table,
          PathUtil.getInputTemplates(this));
    }
    for (int i = 0; i < table.length; i++) {
      if (table[i][0]!="@ENHYDRA_DIR@") // vl
        table[i][1] = PathUtil.compressPathRelativeToProject(this,table[i][1]);
    }
    return table;
  }

  /**
   * Method declaration
   *
   * @param map
   */
  public void setReplacementTable(String[][] table) {

    // replace relative paths in replaceWith column
    for (int i = 0; i < table.length; i++) {
      if (table[i][0]!="@ENHYDRA_DIR@") // vl
        table[i][1] = PathUtil.compressPathRelativeToProject(this,table[i][1]);
    }
    PropUtil.putArrayProperty(this, DEPLOY_TABLE_LENGTH,
                              DEPLOY_TABLE_FIND, DEPLOY_TABLE_REPLACE,
                              table);
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public String getGenerateToDirectory() {
    String genToDir = null;

    if (readBooleanProperty(PropertyKeys.NAME_GENTO, true)) {
      genToDir = getClassOutputPath() + File.separator
               + Constants.DIR_GEN_SOURCE;
    }
    return genToDir;
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public boolean isDeployOverwrite() {
    String in = getProperty(DEPLOY_OVERWRITE);

    return PropUtil.stringToBoolean(in, true);
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public boolean isDodsOverwrite() {
    String in = getProperty(DODS_OVERWRITE);

    return PropUtil.stringToBoolean(in, true);
  }

  /**
   * Method declaration
   *
   *
   * @param b
   */
  public void setDeployOverwrite(boolean b) {
    setProperty(DEPLOY_OVERWRITE, PropUtil.booleanToString(b));
  }

  /**
   * Method declaration
   *
   *
   * @param b
   */
  public void setDodsOverwrite(boolean b) {
    setProperty(DODS_OVERWRITE, PropUtil.booleanToString(b));
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public boolean isDeployInputFilter() {
    String  in = getProperty(DEPLOY_INPUT_FILTER);
    boolean inFilter = false;

    inFilter = PathUtil.isInputInSource(this);
    if (!inFilter) {
      inFilter = PropUtil.stringToBoolean(in, false);
    }
    return inFilter;
  }

  /**
   * Method declaration
   *
   *
   * @param b
   */
  public void setDeployInputFilter(boolean b) {
    boolean inFilter = false;

    inFilter = PathUtil.isInputInSource(this);
    if (inFilter) {
      b = true;
    }
    setProperty(DEPLOY_INPUT_FILTER, PropUtil.booleanToString(b));
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public boolean isDeployStartupRun() {
    String in = getProperty(DEPLOY_STARTUP_RUN);

    return PropUtil.stringToBoolean(in, true);
  }

  /**
   * Method declaration
   *
   *
   * @param b
   */
  public void setDeployStartupRun(boolean b) {
    setProperty(DEPLOY_STARTUP_RUN, PropUtil.booleanToString(b));
  }

  public boolean isDeployStartupJavaReadOnly() {
    return false;
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public boolean isDeployStartupJava() {
    boolean javaStartup = false;
    String  in = getProperty(DEPLOY_STARTUP_JAVA);

    javaStartup = PropUtil.stringToBoolean(in, !isOpenBuild());
    return javaStartup;
  }

  /**
   * Method declaration
   *
   *
   * @param b
   */
  public void setDeployStartupJava(boolean b) {
    setProperty(DEPLOY_STARTUP_JAVA, PropUtil.booleanToString(b));
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public String getDeployBootstrapPath() {
    PathHandle ph = null;
    File       file = null;

    file = PathUtil.getFileRelativeToProject(this, DEPLOY_PATH_BOOTSTRAP);
    ph = PathHandle.createPathHandle(file);
    if (ph.isEmpty()) {
      ph =
          PathHandle.createPathHandle(PathUtil.getDefaultDeployBootstrapPath(this));
    }
    return ph.getPath();
  }

  /**
   * Method declaration
   *
   *
   * @param conf
   */
  public void setDeployBootstrapPath(String path) {
    PathUtil.putFileRelativeToProject(this, DEPLOY_PATH_BOOTSTRAP, path);
  }

  /**
   * Method declaration
   *
   *
   * @param conf
   */
  public void setDodsBootstrapPath(String path) {
    PathUtil.putFileRelativeToProject(this, DODS_PATH_BOOTSTRAP, path);
  }

  public void setArchivePlans(JarPlan[] plans) {
    PropertyWriter writer = null;

    writer = new PropertyWriter();
    writer.write(plans, getArchivePH().getFile());
  }

  public JarPlan[] getArchivePlans(boolean validate) {
    PropertyReader reader = null;
    JarPlan[]      plans = null;

    reader = new PropertyReader();
    reader.setValidate(validate);
    if (getArchivePH().isFile()) {
      try {
        plans = reader.read(getArchivePH().getFile());
      } catch (IOException e) {
        e.printStackTrace(System.err);
        plans = null;
      }
    }
    if (plans == null) {
      plans = reader.read(defaultArchiveProperties());
    }
    return plans;
  }

  public String getAutoDeployFilePath() {
    String       path = new String();
    StringBuffer buf = new StringBuffer();
    PathHandle   ph = null;
    JarPlan[]    plans = new JarPlan[0];
    int          dep = 0;

    path = getProperty(PropertyKeys.AUTO_DEPLOY_FILE);
    ph = PathHandle.createPathHandle(path);
    if (ph.isEmpty()) {
      if (getDeployType() == OtterProject.TYPE_EN3APP) {
        buf.append(PathUtil.getDeployConfPath(this));
        buf.append(File.separator);
        buf.append(getProjectRootName());
        buf.append(".conf");
        path = PathHandle.createPathString(buf.toString());
      } else {
        plans = getArchivePlans(false);
        if (plans.length > 0) {
          path = plans[plans.length - 1].getArchivePath();
        }
      }
    } else {
      path = ph.getPath();
    }
    return path;
  }

  public void setAutoDeployFile(String path) {
    setProperty(PropertyKeys.AUTO_DEPLOY_FILE,
                PathHandle.createPathString(path));
  }

  /**
   * Is deployment desired during project build.
   */
  public boolean isDeployBuild() {
    return readBooleanProperty(DEPLOY_BUILD, false);
  }

  /**
   * Set if deployment is desired during project build.
   */
  public void setDeployBuild(boolean b) {
    writeBooleanProperty(DEPLOY_BUILD, b);
  }

  /**
   * Set if deployment is desired during project build.
   */
  public void setDodsBuild(boolean b) {
    writeBooleanProperty(DODS_BUILD, b);
  }

  /**
   * Is processing shown during deployment.
   */
  public boolean isDeployEcho() {
    return readBooleanProperty(DEPLOY_ECHO, true);
  }

  /**
   * Is processing shown during generating.
   */
  public boolean isDodsEcho() {
    return readBooleanProperty(DODS_ECHO, true);
  }

  /**
   * Set if processing should be shown during deployment.
   */
  public void setDeployEcho(boolean b) {
    writeBooleanProperty(DEPLOY_ECHO, b);
  }

  /**
   * Set if processing should be shown during deployment.
   */
  public void setDodsEcho(boolean b) {
    writeBooleanProperty(DODS_ECHO, b);
  }

  /**
   * Method declaration
   *
   *
   * @param node
   *
   * @return
   */
  public String getSourcePathOf(OtterFileNode node) {
    String out = new String();
    String filePath = node.getFilePath();

    out = PathUtil.getSourcePathOf(this, filePath);
    return out;
  }

  /**
   * Method declaration
   *
   *
   * @param property
   * @param b
   */
  protected void writeBooleanProperty(String property, boolean b) {
    String flag = b ? Boolean.TRUE.toString() : Boolean.FALSE.toString();

    setProperty(property, flag);
  }

  /**
   * Method declaration
   *
   * @param property
   * @param def
   *
   * @return
   */
  protected boolean readBooleanProperty(String property, boolean def) {
    boolean flag = def;
    String  in = getProperty(property);

    return PropUtil.stringToBoolean(in, def);
  }

  public boolean isSelected() {
    return false;
  }

  public void setSelected(boolean b) {}

  public void setException(Throwable e) {}

  public Throwable getException() {
    return null;
  }

  private PathHandle getArchivePH() {
    PathHandle   ph = null;
    StringBuffer buf = new StringBuffer();

    buf.append(getRootPath());
    buf.append(File.separator);
    buf.append("archive.properties");
    ph = PathHandle.createPathHandle(buf.toString());
    return ph;
  }

  private Properties defaultArchiveProperties() {
    Descriptor[]   dd = new Descriptor[0];
    JarPlan        plan = null;
    PropertyWriter writer = null;
    Properties     props = new Properties();
    int            type = getDeployType();

    try {
      switch (type) {
        case TYPE_WEBAPP:
          WarPlan wp = new WarPlan();

          wp.setValidate(false);
          wp.setContentRoot(PathUtil.getDeployContentPath(this));
          dd = new Descriptor[1];
          dd[0] = new Descriptor(true, Descriptor.WEB);
          dd[0].setPath(getSourcePathArray()[0] + "/WEB-INF/web.xml");
          plan = wp;
          break;
        case TYPE_EN3APP:
          plan = new JarPlan();
          plan.setValidate(false);
          break;
//Dusan
/*            case TYPE_SERVICE:
                plan = new JarPlan();
                plan.setValidate(false);
                dd = new Descriptor[1];
                dd[0] = new Descriptor(true, Descriptor.ENHYDRA_SERVICE);
                dd[0].setPath(getSourcePathArray()[0]
                              + "/META-INF/enhydra-service.xml");
                break; */
      }
      if (plan != null) {
        plan.setDescriptors(dd);
        plan.setClassRoot(getClassOutputPath());
        plan.setArchivePath(PathUtil.getDefaultDeployArchivePath(this));
        writer = new PropertyWriter();
        props = writer.write(plan);
      }
    } catch (ArchiveException e) {
      e.printStackTrace(System.err);
      if (props == null) {
        props = new Properties();
      }
    }
    return props;
  }

  /**
   * Method declaration
   *
   * @param p
   */
  public void setDodsParameters(String p) {
    setProperty(NAME_DODS_PARAMETERS, p);
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public String getDodsParameters() {
    return getProperty(NAME_DODS_PARAMETERS);
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public void setDomlFilePath(String path) {
    setProperty(NAME_DOML_FILEPATH, path);
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public String getDomlFilePath() {
    return getProperty(NAME_DOML_FILEPATH);
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public String getEnhydraPath() {
    return getProperty(DODS_ENHYDRA_PATH);
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public String getProjectJavaPath() {
    return getProperty(DODS_PROJECT_JAVA_PATH);
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public void setDodsRootPath(String path) {
    setProperty(NAME_DODS_ROOTPATH, path);
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public String getDodsRootPath() {
    return getProperty(NAME_DODS_ROOTPATH);
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
/*    public void setExstensionParameter(String ext) {
        setProperty(NAME_DODS_EXTENSION, ext);
    } */

   /**
    * Method declaration
    *
    *
    * @return
    */
/*    public String getExstensionParameter() {
        return getProperty(NAME_DODS_EXTENSION);
    } */

   /**
    * Method declaration
    *
    *
    * @return
    */
  public void setDODSGenerator(DODSGenerator gen) {
    generator = gen;
  }
  /**
   * Method declaration
   *
   *
   * @return
   */
  public DODSGenerator getDODSGenerator() {
    return generator;
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public void setDocumentationHtml(String html) {
    setProperty(NAME_DODS_DOCUMENTATION_HTML, html);
  }
  /**
   * Method declaration
   *
   *
   * @return
   */
  public String getDocumentationHtml() {
    return getProperty(NAME_DODS_DOCUMENTATION_HTML);
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public void setDocumentationPdf(String pdf) {
    setProperty(NAME_DODS_DOCUMENTATION_PDF, pdf);
  }
  /**
   * Method declaration
   *
   *
   * @return
   */
  public String getDocumentationPdf() {
    return getProperty(NAME_DODS_DOCUMENTATION_PDF);
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public void setDocumentationXmi(String xmi) {
    setProperty(NAME_DODS_DOCUMENTATION_XMI, xmi);
  }
  /**
   * Method declaration
   *
   *
   * @return
   */
  public String getDocumentationXmi() {
    return getProperty(NAME_DODS_DOCUMENTATION_XMI);
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public void setDocumentationPtl(String ptl) {
    setProperty(NAME_DODS_DOCUMENTATION_PTL, ptl);
  }
  /**
   * Method declaration
   *
   *
   * @return
   */
  public String getDocumentationPtl() {
    return getProperty(NAME_DODS_DOCUMENTATION_PTL);
  }

  /**
   * Method declaration
   *
   *
   * @return
   */
  public void setTemplateSet(String temp) {
    setProperty(NAME_DODS_TEMPLATE_SET, temp);
  }
  /**
   * Method declaration
   *
   *
   * @return
   */
  public String getTemplateSet() {
    return getProperty(NAME_DODS_TEMPLATE_SET);
  }

}