/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.properties;

// ToolBox
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.DataValidationException;
import org.enhydra.tool.common.IconSet;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.configure.ConfigTool;

// AddinCore
import org.enhydra.kelp.common.Backward;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.event.SwingTableSelectionEvent;
import org.enhydra.kelp.common.event.SwingTableSelectionListener;
import org.enhydra.kelp.common.deployer.ReplacementTablePanel;
import org.enhydra.kelp.common.swing.AddinInnerPanel;

// JDK
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.*;
import java.io.File;
import java.util.ResourceBundle;
import javax.swing.*;
import javax.swing.border.TitledBorder;

//
public class TemplateNodePropertyPanel extends AddinInnerPanel {
    static ResourceBundle         res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private LocalRadioListener    radioListener = null;
    private GridBagLayout         layoutMain;
    private JCheckBox             checkOutput;
    private JCheckBox             checkOverwrite;
    private ReplacementTablePanel table;
    private JCheckBox             checkBootstrap;
    private JPanel                panelDeploy;
    private JRadioButton          radioFilter;
    private JRadioButton          radioAll;
    private GridBagLayout         layoutDeploy;

    public TemplateNodePropertyPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public synchronized void addSwingTableSelectionListener(SwingTableSelectionListener l) {
        table.addSwingTableSelectionListener(l);
    }

    public synchronized void removeSwingTableSelectionListener(SwingTableSelectionListener l) {
        table.removeSwingTableSelectionListener(l);
    }

    public void setIconView(boolean b) {
        table.setIconView(b);
    }

    public boolean isIconView() {
        return table.isIconView();
    }

    // override AddinCoreNode
    public void read(OtterNode node) {
        super.read(node);
        String       stringIn = null;
        File         fileIn = null;
        PathHandle   phBoot = null;
        OtterProject project = null;

        table.setNode(node);
        project = getProject();
        phBoot =
            PathHandle.createPathHandle(project.getDeployBootstrapPath());
        if (node instanceof OtterTemplateNode) {
            OtterTemplateNode template = (OtterTemplateNode) node;

            panelDeploy.setVisible(false);
            checkOutput.setSelected(template.isSelected());
            if (phBoot.equals(template.getOutputPath())) {
                checkBootstrap.setSelected(true);
            } else {
                checkBootstrap.setSelected(false);
            }
            remove(panelDeploy);
            enableUI(checkOutput.isSelected());
        } else {
            panelDeploy.setVisible(true);
            checkOutput.setVisible(false);
            checkBootstrap.setVisible(false);
            remove(checkOutput);
            remove(checkBootstrap);
            readFilter();
            enableUI(true);
        }
        checkOverwrite.setSelected(project.isDeployOverwrite());
        table.readProperties();
        invalidate();
    }

    // override AddinCoreNode
    public void write(OtterNode node) throws DataValidationException {
        super.write(node);
        OtterProject project = getProject();

        if (node instanceof OtterTemplateNode) {
            OtterTemplateNode template = (OtterTemplateNode) node;

            template.setSelected(checkOutput.isSelected());
            if (checkBootstrap.isSelected()) {
                project.setDeployBootstrapPath(template.getOutputPath());
            }
        } else {
            if (radioFilter.isEnabled()) {
                project.setDeployInputFilter(radioFilter.isSelected());
            }
        }
        project.setDeployOverwrite(checkOverwrite.isSelected());
        table.writeProperties();
    }

    // override AddinInnerPanel
    public Component[] getFirstFocusComponents() {
        Component[] comps = new Component[7];

        comps[0] = checkOutput;
        comps[1] = checkOverwrite;
        comps[2] = table;
        comps[3] = checkBootstrap;
        comps[4] = panelDeploy;
        comps[5] = radioFilter;
        comps[6] = radioAll;
        return comps;
    }

    //
    //
    protected void readFilter() {
        boolean inFilter = false;

        inFilter = PathUtil.isInputInSource(getProject());
        if (inFilter) {
            radioAll.setEnabled(false);
            radioFilter.setEnabled(false);
        } else {
            radioAll.setEnabled(true);
            radioFilter.setEnabled(true);
            inFilter = getProject().isDeployInputFilter();
        }
        if (inFilter) {
            radioAll.setSelected(false);
            radioFilter.setSelected(true);
        } else {
            radioAll.setSelected(true);
            radioFilter.setSelected(false);
        }
        updateUI();
    }

    //
    //
    private void refreshSelections() {
        getProject().setDeployInputFilter(radioFilter.isSelected());
    }

    private void enableUI(boolean enable) {
        checkOverwrite.setEnabled(enable);
        checkBootstrap.setEnabled(enable);
        table.enableUI(enable);
    }

    private void pmInit() throws Exception {
        ButtonGroup group = null;

        group = new ButtonGroup();
        group.add(radioAll);
        group.add(radioFilter);
        radioListener = new LocalRadioListener();
        radioAll.addItemListener(radioListener);
        radioFilter.addItemListener(radioListener);
        setIconView(false);
    }

    private void jbInit() throws Exception {
        checkOutput =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        checkOverwrite =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        checkBootstrap =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        table =
            (ReplacementTablePanel) Beans.instantiate(getClass().getClassLoader(),
                ReplacementTablePanel.class.getName());
        panelDeploy = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        radioFilter =
            (JRadioButton) Beans.instantiate(getClass().getClassLoader(),
                                             JRadioButton.class.getName());
        radioAll =
            (JRadioButton) Beans.instantiate(getClass().getClassLoader(),
                                             JRadioButton.class.getName());
        layoutDeploy =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        checkOutput.setText(res.getString("Deploy"));
        checkOutput.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                enableUI(checkOutput.isSelected());
            }

        });
        checkOverwrite.setText(res.getString("checkOverwrite_Text"));
        checkBootstrap.setText(res.getString("checkBootstrap_Text"));
        radioFilter.setText(res.getString("radioFilter_Text"));
        radioAll.setSelected(true);
        radioAll.setText(res.getString("radioAll_Text"));

        //
        //
        panelDeploy.setBorder(BorderFactory.createTitledBorder(res.getString("panelDeploy_Border")));
        panelDeploy.setLayout(layoutDeploy);
        panelDeploy.add(radioFilter,
                        new GridBagConstraints(1, 0, 1, 1, 0.1, 0.0,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.HORIZONTAL,
                                               new Insets(1, 5, 1, 10), 10,
                                               0));
        panelDeploy.add(radioAll,
                        new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.HORIZONTAL,
                                               new Insets(1, 10, 1, 5), 10,
                                               0));
        table.setBorder(BorderFactory.createTitledBorder("Template Replacements"));

        //
        this.setLayout(layoutMain);
        this.add(checkOutput,
                 new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(10, 5, 5, 5), 0, 0));
        this.add(checkOverwrite,
                 new GridBagConstraints(0, 1, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(3, 20, 2, 5), 0, 0));
        this.add(panelDeploy,
                 new GridBagConstraints(0, 2, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(4, 15, 4, 5), 0, 1));
        this.add(checkBootstrap,
                 new GridBagConstraints(0, 3, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 20, 3, 5), 0, 0));
        this.add(table,
                 new GridBagConstraints(0, 4, 1, 1, 0.4, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(3, 15, 3, 5), 10, 20));
    }

    //
    private class LocalRadioListener implements ItemListener {
        public void itemStateChanged(ItemEvent e) {
            if (e.getStateChange() == ItemEvent.SELECTED) {
                refreshSelections();
            }
        }

    }

    //
}
