/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.xmlc;

// AddinCore
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.event.WriteEvent;
import org.enhydra.kelp.common.event.WriteListener;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.swing.OutputPanel;

// ToolBox
import org.enhydra.tool.common.ExtensionFilter;
import org.enhydra.tool.common.PathHandle;

// JDK
import javax.swing.*;
import javax.swing.text.BadLocationException;
import java.awt.*;
import java.awt.event.*;
import java.beans.Beans;
import java.io.File;
import java.io.FileWriter;
import java.util.ResourceBundle;

//
public class XMLCOutputPanel extends JPanel implements WriteListener {

    //
    public static ResourceBundle res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private LocalButtonListener  buttonListener = null;
    private LocalCheckListener   checkListener = null;
    private OtterProject         project = null;
    private JPanel               panelFile;
    private JCheckBox            checkSave;
    private OutputPanel          outputPanel;
    private JTextField           textFile;
    private JButton              buttonBrowse;
    private GridBagLayout        layoutFile;
    private GridBagLayout        layoutMain;
    private int                  waitCount = 0;

    /**
     * Constructor declaration
     *
     */
    public XMLCOutputPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Method declaration
     *
     *
     * @param event
     */
    public void onWrite(WriteEvent event) {
        outputPanel.onWrite(event);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public OtterProject getProject() {
        return project;
    }

    /**
     * Method declaration
     *
     *
     * @param p
     */
    public void setProject(OtterProject p) {
        project = p;
        String outPath = project.getOutputFilename();

        if ((outPath == null) || (outPath.trim().length() == 0)) {
            checkSave.setSelected(false);
            textFile.setText(new String());
        } else {
            textFile.setText(PathHandle.createPathString(outPath));
            checkSave.setSelected(true);
        }
        outputPanel.setProject(project);
    }

    public void clearOutput() {
        outputPanel.clearOutput();
    }

    //
    // PROTECTED
    //

    /**
     * Method declaration
     *
     */
    protected void scrollToBottom() {
        outputPanel.scrollToBottom();
    }

    //
    // PRIVATE
    //
    private void buttonFileAction() {
        JFileChooser    chooser = null;
        ExtensionFilter filter = null;
        PathHandle      path = null;
        File            file = null;
        String          fileDir = new String();
        int             choice = -1;

        //
        filter = new ExtensionFilter();
        filter.addExtension(Constants.TYPE_TXT);
        filter.setDescriptionTitle(res.getString("Kelp_log"));

        //
        chooser = new JFileChooser();
        if (textFile.getText().trim().length() == 0) {
            file = new File(project.getRootPath() + File.separator
                            + Constants.FILE_KELP_TXT);
        } else {
            file = new File(textFile.getText());
        }
        chooser.setSelectedFile(file);
        chooser.setFileFilter(filter);
        chooser.setDialogTitle(res.getString("Select_log"));
        chooser.setApproveButtonText(res.getString("OK"));
        choice = chooser.showOpenDialog(this);

        //
        this.requestFocus();
        buttonBrowse.requestFocus();

        //
        if (choice == JFileChooser.APPROVE_OPTION
                && chooser.getSelectedFile() != null) {
            path = PathHandle.createPathHandle(chooser.getSelectedFile());
            textFile.setText(path.getPath());
            project.setOutputFilename(path.getPath());
        }

        //
        chooser.removeAll();
        chooser = null;
    }

    /**
     * Method declaration
     *
     */
    private void pmInit() {
        buttonListener = new LocalButtonListener();
        buttonBrowse.addActionListener(buttonListener);
        buttonBrowse.setEnabled(false);
        checkListener = new LocalCheckListener();
        checkSave.addItemListener(checkListener);
        checkSave.setSelected(false);
    }

    /**
     * Method declaration
     *
     *
     * @throws Exception
     */
    private void jbInit() throws Exception {
        panelFile = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                               JPanel.class.getName());
        outputPanel =
            (OutputPanel) Beans.instantiate(getClass().getClassLoader(),
                                            OutputPanel.class.getName());
        checkSave = (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                                  JCheckBox.class.getName());
        textFile = (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                                  JTextField.class.getName());
        buttonBrowse =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        layoutFile =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        buttonBrowse.setText(res.getString("Browse"));
        panelFile.setLayout(layoutFile);
        checkSave.setText(res.getString("Output_to_log"));
        textFile.setBackground(SystemColor.controlLtHighlight);
        textFile.setEnabled(false);
        textFile.setPreferredSize(new Dimension(150, 21));
        panelFile.add(checkSave,
                      new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                             GridBagConstraints.WEST,
                                             GridBagConstraints.BOTH,
                                             new Insets(5, 5, 5, 5), 14, 0));
        panelFile.add(textFile,
                      new GridBagConstraints(1, 0, 1, 1, 0.5, 0.5,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(5, 5, 5, 5), 14, 0));
        panelFile.add(buttonBrowse,
                      new GridBagConstraints(2, 0, 1, 1, 0.5, 0.5,
                                             GridBagConstraints.WEST,
                                             GridBagConstraints.NONE,
                                             new Insets(5, 5, 5, 5), 6, 0));
        this.setLayout(layoutMain);
        this.add(outputPanel,
                 new GridBagConstraints(0, 0, 1, 1, 0.75, 0.75,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.BOTH,
                                        new Insets(8, 12, 0, 0), 0, 0));
        this.add(panelFile,
                 new GridBagConstraints(0, 1, 1, 1, 0.25, 0.25,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(0, 12, 0, 0), 0, 0));
    }

    /**
     */
    private class LocalCheckListener implements ItemListener {
        public void itemStateChanged(ItemEvent event) {
            Object source = event.getSource();

            if (source == checkSave) {
                if (checkSave.isSelected()) {
                    buttonBrowse.setEnabled(true);
                } else {
                    buttonBrowse.setEnabled(false);
                    textFile.setText(new String());
                    if (project == null) {

                        // done
                    } else {
                        project.setOutputFilename(new String());
                    }
                }
            }
        }

    }

    /**
     */
    private class LocalButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            if (source == buttonBrowse) {
                buttonFileAction();
            }
        }

    }
}
