/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Lisa Reese
 *
 */

package org.enhydra.kelp.forte.node;

import java.io.IOException;
import java.util.Vector;

import org.openide.actions.*;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.*;
import org.openide.util.NbBundle;
import org.openide.util.actions.SystemAction;

import java.util.Enumeration;
import java.io.*;

/** Recognizes some set of files in the Repository as belonging together.
 *
 * @author rees0234
 */
public class XMLCDataLoader extends MultiFileLoader {

    static final long serialVersionUID = -5042424699146221934L;

    public static final String PROP_XMLC_EXTENSIONS = "xmlc.extensions";


    public XMLCDataLoader ()
    {
        this (XMLCDataObject.class);
    }

    public XMLCDataLoader (Class recognizedObject)
    {
         super (recognizedObject);
    }

    protected void initialize () {

        setDisplayName (NbBundle.getMessage (XMLCDataLoader.class, "LBL_XMLCloader"));

        setActions (new SystemAction[] {
                        SystemAction.get (ViewAction.class),
                        SystemAction.get (OpenAction.class),
                        SystemAction.get (FileSystemAction.class),
                        null,
//                        SystemAction.get (CompileAction.class),
//                        SystemAction.get (EnhydraAction.class),
//                        null,
                        SystemAction.get (CutAction.class),
                        SystemAction.get (CopyAction.class),
                        SystemAction.get (PasteAction.class),
                        null,
                        SystemAction.get (DeleteAction.class),
                        SystemAction.get (RenameAction.class),
                        null,
                        SystemAction.get (SaveAsTemplateAction.class),
//                        null,
//                        SystemAction.get (ToolsAction.class),
                        SystemAction.get (PropertiesAction.class),
                    });

        super.initialize();

    }

    protected FileObject findPrimaryFile (FileObject fo) {
        // This can of course be much more sophisticated.
        Enumeration extSet = getExtensions().extensions();
        while (extSet.hasMoreElements())
        {
            String ext = (String)extSet.nextElement();
            if (fo.hasExt (ext))
                return fo;
        }
//        else if (fo.hasExt (SECONDARY_EXT))
//            return FileUtil.findBrother (fo, MAIN_EXT);
//        else
            return null;
    }

    protected MultiDataObject createMultiObject (FileObject primaryFile)
    throws DataObjectExistsException, IOException {
        return new XMLCDataObject (primaryFile, this);
    }

    protected MultiDataObject.Entry createPrimaryEntry (MultiDataObject obj, FileObject primaryFile) {
        // Entry for the important file: by default, is preserved during all operations.
        return new FileEntry (obj, primaryFile);
    }

    /** @return The list of extensions this loader recognizes
    *
    */
    public ExtensionList getExtensions () {
        ExtensionList extensions = null;
        ForteProject project = new ForteProject();
        if (project != null){
            String exts[] = project.getDocTypes();
	    if (exts != null) {
                //yuk - I don't like this.
                extensions = new ExtensionList();
                for (int i = 0; i < exts.length; i++)
                    extensions.addExtension(exts[i]);
	        putProperty(PROP_XMLC_EXTENSIONS, extensions, false);
            }
        }
        else
            extensions = (ExtensionList)getProperty(PROP_XMLC_EXTENSIONS);
        return extensions;
    }

    /** Sets the extension list for this data loader.
    * @param ext new list of extensions.
    */
    public void setExtensions(ExtensionList ext) {
        putProperty(PROP_XMLC_EXTENSIONS, ext, true);
        ForteProject project = new ForteProject();
        if (project != null){
            Vector v = new Vector();
            Enumeration e = ext.extensions();
            while (e.hasMoreElements())
                v.add(e.nextElement());
            String exts[] = new String[0];
            exts = (String[])v.toArray(exts);
            project.setDocTypes(exts);
        }
    }

    protected MultiDataObject.Entry createSecondaryEntry (MultiDataObject obj, FileObject secondaryFile) {
        secondaryFile.setImportant (false);
        return new FileEntry.Numb (obj, secondaryFile); // discard whenever needed
    }

}
