/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Lisa Reese
 *
 */
package org.enhydra.kelp.forte.actions;

// OpenIDE
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.CallableSystemAction;
import org.openide.TopManager;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.src.ClassElement;
import org.openide.DialogDescriptor;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.JarFileSystem;
import org.openide.filesystems.Repository;

// ToolBox
import org.enhydra.tool.common.SwingUtil;
import org.enhydra.tool.common.event.HelpEvent;
import org.enhydra.tool.common.event.HelpListener;
import org.enhydra.tool.ToolBoxInfo;

// AddinCore
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.dods.CoreDodsTool;
import org.enhydra.kelp.common.dods.DodsButtonPanel;
import org.enhydra.kelp.common.dods.DodsButtonListener;
import org.enhydra.kelp.common.node.OtterFileNode;

// AddinForte
import org.enhydra.kelp.forte.node.ForteProject;
import org.openide.filesystems.LocalFileSystem;

// JDK
import java.awt.Dialog;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.net.URL;
import java.net.MalformedURLException;
import java.io.IOException;
import java.beans.PropertyVetoException;
import javax.swing.JOptionPane;

// mn 08.06.2002
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
/**
 * Action sensitive to some cookie that does something useful.
 *
 * @author  rees0234
 */
public class DODSAction extends CallableSystemAction implements HelpListener {
    private static String  anchor = "dodswizard";

    // implements HelpListener
    public void onHelp(HelpEvent event) {
        URL          url = null;
        String[]     home = new String[2];
        home[0] = System.getProperty("netbeans.home");
        home[1] = System.getProperty("netbeans.user");
        StringBuffer buf = new StringBuffer();
        buf.append(KelpInfo.getAddinHelpURL(home));
        buf.append('#').append(anchor);
        try {
          url = new URL(buf.toString());
          TopManager.getDefault().showUrl(url);
        } catch (MalformedURLException e) {
          e.printStackTrace(System.err);
        }
    }

    public void performAction() {
        // do work based on the current node selection, e.g.:
        // SourceCookie cookie = (SourceCookie) nodes[0].getCookie (SourceCookie.class);
        // etc.
      ForteProject project = new ForteProject();
      project.setEnhydraPath();

      System.setProperty("DODS_HOME", project.getEnhydraPath()+
        File.separator+"dods");

        String[] options = DodsButtonPanel.getOptions();
        CoreDodsTool tool = new CoreDodsTool();
        DodsButtonListener listener = new DodsButtonListener(tool);
        tool.getInnerPanel().initPreferredSize();

        tool.setProject(project);


        DialogDescriptor dscr = new DialogDescriptor(tool.getInnerPanel(),
                tool.getTitle(), true, options, options[1],
                DialogDescriptor.BOTTOM_ALIGN, HelpCtx.DEFAULT_HELP,
                listener);
        Dialog dlg = TopManager.getDefault().createDialog(dscr);
        tool.addHelpListener(this);
        dlg.show();

        //add enhydra stuff to classpath:
        try {
          Repository rep = TopManager.getDefault().getRepository();

          String target = null;
          StringBuffer buf = new StringBuffer();
          buf.append(ToolBoxInfo.getEnhydraRoot()).append(File.separatorChar).append("lib").
          append(File.separatorChar);
          String base = buf.toString();
          buf.delete(0, buf.length());
          buf.append(base).append("enhydra.jar");
          target = buf.toString();
          buf.delete(0, buf.length());
          {
            JarFileSystem jar = new JarFileSystem();
            jar.setJarFile(new File(target));
            rep.addFileSystem(jar);
          }
          buf.append(ToolBoxInfo.getEnhydraRoot()).append(File.separatorChar).append("dods").
          append(File.separatorChar).append("lib").append(File.separatorChar);
          base = buf.toString();
          buf.delete(0, buf.length());
          buf.append(base).append("dods.jar");
          target = buf.toString();
          buf.delete(0, buf.length());
          {
            JarFileSystem jar = new JarFileSystem();
            jar.setJarFile(new File(target));
            rep.addFileSystem(jar);
          }
          buf.append(ToolBoxInfo.getEnhydraRoot()).append(File.separatorChar).append("lib").
            append(File.separatorChar);
          base = buf.toString();
          buf.delete(0, buf.length());
          buf.append(base).append("core.jar");
          target = buf.toString();
          buf.delete(0, buf.length());
          {
            JarFileSystem jar = new JarFileSystem();
            jar.setJarFile(new File(target));
            rep.addFileSystem(jar);
          }
          buf.append(ToolBoxInfo.getEnhydraRoot()).append(File.separatorChar).append("lib").
            append(File.separatorChar);
          base = buf.toString();
          buf.delete(0, buf.length());
          buf.append(base).append("util.jar");
          target = buf.toString();
          buf.delete(0, buf.length());
          {
            JarFileSystem jar = new JarFileSystem();
            jar.setJarFile(new File(target));
            rep.addFileSystem(jar);
          }
          buf.append(ToolBoxInfo.getEnhydraRoot()).append(File.separatorChar).append("dods").
          append(File.separatorChar).append("lib").append(File.separatorChar);
          base = buf.toString();
          buf.delete(0, buf.length());
          buf.append(base).append("dods-runtime.jar");
          target = buf.toString();
          buf.delete(0, buf.length());
          {
            JarFileSystem jar = new JarFileSystem();
            jar.setJarFile(new File(target));
            rep.addFileSystem(jar);
          }
          buf.append(ToolBoxInfo.getEnhydraRoot()).append(File.separatorChar).
          append("lib").append(File.separatorChar);
          base = buf.toString();
          buf.delete(0, buf.length());
          buf.append(base).append("log4j.jar");
          target = buf.toString();
          buf.delete(0, buf.length());
          {
            JarFileSystem jar = new JarFileSystem();
            jar.setJarFile(new File(target));
            rep.addFileSystem(jar);
          }

          //adding dods root path into the filesystems
          String s = tool.getOption()+"";
            buf.append(project.getDodsRootPath());
            target = buf.toString();
            buf.delete(0, buf.length());
            {
              LocalFileSystem fs = new LocalFileSystem();
              fs.setRootDirectory(new File(target));
              rep.addFileSystem(fs);
            }
         } catch (IOException e) {
            System.err.println(e);
        } catch (PropertyVetoException e) {
            System.err.println(e);
        }
    }

    public String getName() {
        return CoreDodsTool.getDefaultTitle();
    }

    public boolean isEnabled()
    {
        return (new ForteProject().getRootFolder() != null);
    }


    protected String iconResource() {
        return "smallicon.gif";
    }

    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;

        // If you will provide context help then use:
        // return new HelpCtx (DeployementAction.class);
    }

    /**
     * Perform extra initialization of this action's singleton.
     * PLEASE do not use constructors for this purpose!
     */
    protected void initialize() {
        // mn 08.06.2002
        super.initialize();
        putProperty(javax.swing.Action.SHORT_DESCRIPTION,
                    NbBundle.getMessage(DeploymentAction.class,
                                        "HINT_DODS_Action"));
    }

}
