package org.enhydra.kelp.eclipse.enhydraWizard;

import java.io.File;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.help.WorkbenchHelp;
import org.eclipse.ui.internal.IHelpContextIds;
import org.eclipse.ui.internal.WorkbenchPlugin;

/**
 * @author Tweety
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class NewEnhydraWizardPage extends WizardPage {

	private boolean useDefaults = true;

	// initial value stores
	private String initialProjectFieldValue;
	private IPath initialLocationFieldValue;
	
	// the value the user has entered
	private String customLocationFieldValue;
	
	// widgets
	private Text projectNameField;
	private Text locationPathField;
	private Label locationLabel;
	private Button browseButton;
	static private String prjRootValue = null;
	private Listener nameModifyListener = new Listener() {
		public void handleEvent(Event e) {
			setLocationForSelection();
			setPageComplete(validatePage());
		}
	};

	private Listener locationModifyListener = new Listener() {
		public void handleEvent(Event e) {
			setPageComplete(validatePage());
		}
	};

	// constants
	private static final int SIZING_TEXT_FIELD_WIDTH = 250;
	private static final int SIZING_INDENTATION_WIDTH = 10;


	/**
	 * Constructor for NewEnhydraWizardPage.
	 * @param pageName
	 */
	public NewEnhydraWizardPage(String pageName) {
		super(pageName);
	
		setPageComplete(false);
		initialLocationFieldValue = Platform.getLocation();
		customLocationFieldValue = "";
	}

	/**
	 * Constructor for NewEnhydraWizardPage.
	 * @param pageName
	 * @param title
	 * @param titleImage
	 */
	public NewEnhydraWizardPage(
		String pageName,
		String title,
		ImageDescriptor titleImage) {
		super(pageName, title, titleImage);
		
		setPageComplete(false);
		initialLocationFieldValue = Platform.getLocation();
		customLocationFieldValue = "";
	}

	/** (non-Javadoc)
	 * Method declared on IDialogPage.
	 */
	public void createControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.NULL);
	
		WorkbenchHelp.setHelp(composite, IHelpContextIds.NEW_PROJECT_WIZARD_PAGE);
		
		composite.setLayout(new GridLayout());
		composite.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		createProjectNameGroup(composite);
		createProjectLocationGroup(composite);
		validatePage();
		// Show description on opening
		setErrorMessage(null);
		setMessage(null);
		setControl(composite);
	}
	/**
	 * Creates the project location specification controls.
	 *
	 * @param parent the parent composite
	 */
	private final void createProjectLocationGroup(Composite parent) {
	
		// project specification group
		Composite projectGroup = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		projectGroup.setLayout(layout);
		projectGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
	 
		// new project label
		Label projectContentsLabel = new Label(projectGroup, SWT.NONE);
		projectContentsLabel.setText(NewEnhydraWizardMessages.getString("NewEnhydraWizardPage.projectContentsLabel")); 
	
		GridData labelData = new GridData();
		labelData.horizontalSpan = 3;
		projectContentsLabel.setLayoutData(labelData);
	
		final Button useDefaultsButton = new Button(projectGroup, SWT.CHECK | SWT.RIGHT);
		useDefaultsButton.setText(NewEnhydraWizardMessages.getString("NewEnhydraWizardPage.useDefaultLabel")); 
		useDefaultsButton.setSelection(useDefaults);
	
		GridData buttonData = new GridData();
		buttonData.horizontalSpan = 3;
		useDefaultsButton.setLayoutData(buttonData);
	
		createUserSpecifiedProjectLocationGroup(projectGroup, !useDefaults);
	
		SelectionListener listener = new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				useDefaults = useDefaultsButton.getSelection();
				browseButton.setEnabled(!useDefaults);
				locationPathField.setEnabled(!useDefaults);
				locationLabel.setEnabled(!useDefaults);
				if (useDefaults) {
					customLocationFieldValue = locationPathField.getText();
					setLocationForSelection();
				} else {
					locationPathField.setText(customLocationFieldValue);
				}
			}
		};
		useDefaultsButton.addSelectionListener(listener);
	}
	/**
	 * Creates the project name specification controls.
	 *
	 * @param parent the parent composite
	 */
	private final void createProjectNameGroup(Composite parent) {
		// project specification group
		Composite projectGroup = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		projectGroup.setLayout(layout);
		projectGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
	
		// new project label
		Label projectLabel = new Label(projectGroup, SWT.NONE);
		projectLabel.setText(NewEnhydraWizardMessages.getString("NewEnhydraWizardPage.nameLabel")); 
	
		// new project name entry field
		projectNameField = new Text(projectGroup, SWT.BORDER);
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = SIZING_TEXT_FIELD_WIDTH;
		projectNameField.setLayoutData(data);
	
		// Set the initial value first before listener
		// to avoid handling an event during the creation.
		if (initialProjectFieldValue != null)
			projectNameField.setText(initialProjectFieldValue);
		projectNameField.addListener(SWT.Modify, nameModifyListener);
	}
	/**
	 * Creates the project location specification controls.
	 *
	 * @param projectGroup the parent composite
	 * @param boolean - the initial enabled state of the widgets created
	 */
	private void createUserSpecifiedProjectLocationGroup(Composite projectGroup, boolean enabled) {
	
		// location label
		locationLabel = new Label(projectGroup,SWT.NONE);
		locationLabel.setText(NewEnhydraWizardMessages.getString("NewEnhydraWizardPage.locationLabel")); 
		locationLabel.setEnabled(enabled);
	
		// project location entry field
		locationPathField = new Text(projectGroup, SWT.BORDER);
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = SIZING_TEXT_FIELD_WIDTH;
		locationPathField.setLayoutData(data);
		locationPathField.setEnabled(enabled);
	
		// browse button
		browseButton = new Button(projectGroup, SWT.PUSH);
		browseButton.setText(NewEnhydraWizardMessages.getString("NewEnhydraWizardPage.browseLabel")); 
		browseButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				handleLocationBrowseButtonPressed();
			}
		});
	
		browseButton.setEnabled(enabled);
	
		// Set the initial value first before listener
		// to avoid handling an event during the creation.
		if (initialLocationFieldValue != null)
			locationPathField.setText(initialLocationFieldValue.toOSString());
		locationPathField.addListener(SWT.Modify, locationModifyListener);
	}
	/**
	 * Returns the current project location path as entered by 
	 * the user, or its anticipated initial value.
	 * Note that if the default has been returned the path
	 * in a project description used to create a project
	 * should not be set.
	 *
	 * @return the project location path, its anticipated initial value, or <code>null</code>
	 *   if no project location path is known
	 */
	public IPath getLocationPath() {
		if (useDefaults)
			return initialLocationFieldValue;
			
		return new Path(getProjectLocationFieldValue());
	}
	/**
	 * Creates a project resource handle for the current project name field value.
	 * <p>
	 * This method does not create the project resource; this is the responsibility
	 * of <code>IProject::create</code> invoked by the new project resource wizard.
	 * </p>
	 *
	 * @return the new project resource handle
	 */
	public IProject getProjectHandle() {
		return ResourcesPlugin.getWorkspace().getRoot().getProject(getProjectName());
	}
	/**
	 * Returns the current project name as entered by the user, or its anticipated
	 * initial value.
	 *
	 * @return the project name, its anticipated initial value, or <code>null</code>
	 *   if no project name is known
	 */
	public String getProjectName() {
		if (projectNameField == null)
			return initialProjectFieldValue;
			
		return getProjectNameFieldValue();
	}
	/**
	 * Returns the value of the project name field
	 * with leading and trailing spaces removed.
	 * 
	 * @return the project name in the field
	 */
	private String getProjectNameFieldValue() {
		if (projectNameField == null)
			return ""; 
		else
			return projectNameField.getText().trim();
	}
	
	 
	/**
	 * Returns the value of the project location field
	 * with leading and trailing spaces removed.
	 * 
	 * @return the project location directory in the field
	 */
	private String getProjectLocationFieldValue() {
		if (locationPathField == null)
			return ""; 
		else
			return locationPathField.getText().trim();
	}
	/**
	 *	Open an appropriate directory browser
	 */
	private void handleLocationBrowseButtonPressed() {
		DirectoryDialog dialog = new DirectoryDialog(locationPathField.getShell());
		dialog.setMessage(NewEnhydraWizardMessages.getString("NewEnhydraWizardPage.directoryLabel")); 
		
		String dirName = getProjectLocationFieldValue();
		if (!dirName.equals("")) { 
			File path = new File(dirName);
			if (path.exists())
				dialog.setFilterPath(new Path(dirName).toOSString());
		}
		
		String selectedDirectory = dialog.open();
		if (selectedDirectory != null) {
			customLocationFieldValue = selectedDirectory;
			locationPathField.setText(customLocationFieldValue);
		}
	}
	/**
	 * Sets the initial project name that this page will use when
	 * created. The name is ignored if the createControl(Composite)
	 * method has already been called. Leading and trailing spaces
	 * in the name are ignored.
	 * 
	 * @param name initial project name for this page
	 */
	public void setInitialProjectName(String name) {
		if (name == null)
			initialProjectFieldValue = null;
		else
			initialProjectFieldValue = name.trim();
	}
	/**
	 * Set the location to the default location if we are set to useDefaults.
	 */
	private void setLocationForSelection() {
		if (useDefaults) {
			IPath defaultPath = Platform.getLocation().append(getProjectNameFieldValue());
			locationPathField.setText(defaultPath.toOSString());
		}
	}
	/**
	 * Returns whether this page's controls currently all contain valid 
	 * values.
	 *
	 * @return <code>true</code> if all controls are valid, and
	 *   <code>false</code> if at least one is invalid
	 */
	private boolean validatePage() {
		IWorkspace workspace = WorkbenchPlugin.getPluginWorkspace();
	
		String projectFieldContents = getProjectNameFieldValue();
		if (projectFieldContents.equals("")) { 
			setErrorMessage(null);
			setMessage(NewEnhydraWizardMessages.getString("NewEnhydraWizardPage.projectNameEmpty")); 
			return false;
		}
		
		IStatus nameStatus =
			workspace.validateName(projectFieldContents, IResource.PROJECT);
		if (!nameStatus.isOK()) {
			setErrorMessage(nameStatus.getMessage());
			return false;
		}
	
		String locationFieldContents = getProjectLocationFieldValue();
		
		if (locationFieldContents.equals("")) { 
			setErrorMessage(null);
			setMessage(NewEnhydraWizardMessages.getString("NewEnhydraWizardPage.projectLocationEmpty")); 
			return false;
		}
		//Dacha add check end of project root
		if (!(locationFieldContents.endsWith("\\"+projectFieldContents) || locationFieldContents.endsWith("/"+projectFieldContents))) { 
			setErrorMessage(null);
			setMessage(NewEnhydraWizardMessages.getString("NewEnhydraWizardPage.projectRootNotEndsName")); 
			return false;
		}
		//end Dacha
		IPath path = new Path("");
		if (!path.isValidPath(locationFieldContents)) {
			setErrorMessage(NewEnhydraWizardMessages.getString("NewEnhydraWizardPage.locationError")); 
			return false;
		}
		if (!useDefaults && Platform.getLocation().isPrefixOf(new Path(locationFieldContents))) {
			setErrorMessage(NewEnhydraWizardMessages.getString("NewEnhydraWizardPage.defaultLocationError")); 
			return false;
		}
	 
	
		if (getProjectHandle().exists()) {
			setErrorMessage(NewEnhydraWizardMessages.getString("NewEnhydraWizardPage.projectExistsMessage")); 
			return false;
		}
	
		setErrorMessage(null);
		setMessage(null);
		return true;
	}
	
	/*
	 * see @DialogPage.setVisible(boolean)
	 */
	public void setVisible(boolean visible) {
		super.setVisible(visible);
		if(visible)
			projectNameField.setFocus();
	}

}
