/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.jbuilder.wizard;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.IconSet;
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.codegen.ValidationException;
import org.enhydra.tool.codegen.GeneratorException;
import org.enhydra.tool.codegen.wizard.CodeGenPanel;

// JBuilder imports
import com.borland.jbuilder.JBuilderHelp;
import com.borland.jbuilder.build.JavaBuildPropertyGroup;
import com.borland.jbuilder.node.JBProject;
import com.borland.jbuilder.node.HTMLFileNode;
import com.borland.jbuilder.paths.JDKPathSet;
import com.borland.jbuilder.paths.PathSetManager;
import com.borland.jbuilder.wizard.project.JBProjectWizard;
import com.borland.primetime.help.ZipHelpBook;
import com.borland.primetime.help.ZipHelpTopic;
import com.borland.primetime.ide.Browser;
import com.borland.primetime.ide.BrowserIcons;
import com.borland.primetime.node.Node;
import com.borland.primetime.node.Project;
import com.borland.primetime.node.DuplicateNodeException;
import com.borland.primetime.util.VetoException;
import com.borland.primetime.vfs.Url;
import com.borland.primetime.wizard.BasicWizard;
import com.borland.primetime.wizard.BasicWizardPage;
import com.borland.primetime.wizard.Wizard;
import com.borland.primetime.wizard.WizardAction;
import com.borland.primetime.wizard.WizardDialog;
import com.borland.primetime.wizard.WizardHost;
import com.borland.primetime.wizard.WizardManager;
import com.borland.primetime.wizard.WizardPage;

// AddinCore imports
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.ResUtil;
import org.enhydra.kelp.common.node.OtterFolderNode;
import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.codegen.ProjectGenUtil;
import org.enhydra.kelp.common.codegen.CodeGenException;

// AddinJBuilder imports
import org.enhydra.kelp.jbuilder.node.PrimeProject;
import org.enhydra.kelp.jbuilder.node.PrimeProject;

// Standard imports
import java.awt.BorderLayout;
import java.io.File;
import java.util.Properties;
import java.util.ArrayList;
import java.util.Iterator;
import javax.swing.JOptionPane;
import java.util.ResourceBundle;
import org.enhydra.tool.codegen.ProjectOptionSet;
import java.net.URL;
import com.borland.primetime.node.FileNode;
import com.borland.primetime.vfs.VFS;
import com.borland.primetime.vfs.Filesystem;
import org.enhydra.kelp.ant.node.AntProject;

//
abstract public class AbstractCodeGenWizard extends AbstractBasicWizard {
    static ResourceBundle res =
        ResourceBundle.getBundle("org.enhydra.kelp.jbuilder.Res");    // nores

    //
    private ProjectGenUtil    util = null;
    private JBProject     nativeProject = null;

    public abstract String getHelpTopic();
    public abstract String getDisplayName();

    /**
     * Method declaration
     *
     *
     * @param currentPage
     * @param host
     */
    public void help(WizardPage wizardPage, WizardHost wizardHost) {
        ZipHelpTopic topic = null;
        ZipHelpBook  book = null;

        book = JBuilderHelp.createBook(Constants.KELP);
        topic = new ZipHelpTopic(book, getHelpTopic());
        topic.show(wizardPage.getPageComponent(wizardHost));
    }

    /**
     * Method declaration
     *
     *
     * @param host
     *
     * @return
     */
    abstract public WizardPage invokeWizard(WizardHost host);

    /**
     * Method declaration
     *
     * @param host
     *
     * @return
     */
    public WizardPage invokeCodeGenWizard(WizardHost host, ProjectGenUtil util) {
        WizardPage wizardPage = null;
        boolean    invoke = true;

        this.util = util;
        if (!isProjectEmpty()) {
            StringBuffer buf = new StringBuffer();
            int          choice = JOptionPane.CANCEL_OPTION;

            buf.append(res.getString("NotEmptyWarning1"));
            buf.append(res.getString("NotEmptyWarning2"));
            buf.append(res.getString("NotEmptyWarning3"));
            choice =
                JOptionPane.showConfirmDialog(host.getDialogParent(),
                                              buf.toString(),
                                              ResUtil.format(res.getString("_0_Wizard_Error"), getDisplayName()),
                                              JOptionPane.WARNING_MESSAGE);
            if (choice == JOptionPane.CANCEL_OPTION) {
                invoke = false;
            }
        }

        if (invoke) {
            try {
                setWizardTitle(ResUtil.format(res.getString("_0_Wizard"),
                                              getDisplayName()));
                if ((nativeProject != null)
                        && (nativeProject instanceof JBProject)) {
                    // always create build script files:
                    try {
                      util.getCodeGen().getSelection().getOptionSet().
                          lookup(ProjectOptionSet.NOCLI).setValue(false);
                    }
                    catch (Exception ex) {
                      ex.printStackTrace();
                    }
                    // length-1: without 3. page with build script file question
                    for (int i = 0; i < util.getWizardPanels().length-1; i++) {
                        LocalWizardPage page = new LocalWizardPage();
                        page.setPanel(util.getWizardPanels()[i]);
                        addWizardPage(page);
                    }
                    wizardPage = super.invokeWizard(host);
                }
            } catch (CodeGenException e) {
                JOptionPane.showMessageDialog(host.getDialogParent(),
                                              e.getMessage(),
                                              ResUtil.format(res.getString("_0_Wizard_Error"), getDisplayName()),
                                              JOptionPane.WARNING_MESSAGE);
            }
        }
        return wizardPage;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    private boolean isProjectEmpty() {
        Node[]  nodes = null;
        boolean empty = true;

        if (nativeProject != null) {
            nodes = nativeProject.getChildren();
        }
        if (nodes != null) {
            if (nodes.length > 1) {
                empty = false;
            } else if (nodes.length == 1) {
                if (!(nodes[0] instanceof HTMLFileNode)) {
                    empty = false;
                }
            }
        }
        return empty;
    }

    /**
     * Method declaration
     *
     *
     * @param host
     */
    protected OtterProject initProject(WizardHost host) {
        OtterProject otterProj = null;

        if (host.getBrowser().getActiveProject() == null) {
            nativeProject = null;
        } else if (host.getBrowser().getActiveProject()
                   instanceof JBProject) {
            nativeProject = (JBProject) host.getBrowser().getActiveProject();
        }
        if (nativeProject == null) {
            WizardDialog    dlg = null;
            JBProjectWizard wiz = null;

            wiz = new JBProjectWizard();
            dlg = new WizardDialog(host.getBrowser(), wiz);
            dlg.show();
            if (host.getBrowser().getActiveProject() == null) {
                nativeProject = null;
            } else if (host.getBrowser().getActiveProject()
                       instanceof JBProject) {
                nativeProject =
                    (JBProject) host.getBrowser().getActiveProject();
            }
        }
        if (nativeProject != null) {
            if (nativeProject instanceof JBProject) {
                otterProj = new PrimeProject((JBProject) nativeProject);
            }
        }
        return otterProj;
    }

    /**
     * Method declaration
     *
     *
     * @throws VetoException
     */
    public WizardPage finish(WizardPage currentPage,
                             WizardHost host) throws VetoException {
        JDKPathSet jdk = null;
        boolean    continueBuild = true;

        jdk = getTargetJDK();
        if (currentPage instanceof BasicWizardPage) {
            BasicWizardPage basicPage = (BasicWizardPage) currentPage;

            basicPage.checkPage();
        }
        if (nativeProject != null && util != null) {
            try {
                if (getSteps().length > 0) {
                  util.getGenerator().setAddinSteps(getSteps());
                }
                if (jdk != null) {
                    String javaPath = new String();

                    javaPath =
                        jdk.getHomePath().getFileObject().getAbsolutePath();
                    util.getGenerator().setJavaPath(javaPath);
                }
                util.generate();
                nativeSetupProject(jdk);

                setProjectProperties(host);

                StringBuffer buf = new StringBuffer();
                buf.append(ResUtil.format(res.getString("AddedFiles"),
                                          getDisplayName()));
                buf.append('\n');
                buf.append(Constants.TAB4);
                buf.append(nativeProject.getDisplayName());
                buf.append('\n');
                buf.append(res.getString("ReferReadme1"));
                buf.append(ResUtil.format(res.getString("ReferReadme2"),
                                          getDisplayName()));
                JOptionPane.showMessageDialog(host.getDialogParent(),
                                              buf.toString());
                openReadme(host.getBrowser());
            } catch (CodeGenException e) {
                JOptionPane.showMessageDialog(this.wizardHost.getDialogParent(),
                                              e.getMessage(),
                                              ResUtil.format(res.getString("_0_Wizard_Error"), getDisplayName()),
                                              JOptionPane.WARNING_MESSAGE);
                throw new VetoException();
            }
        }
        return null;
    }

    private void setProjectProperties(WizardHost host) {
      AntProject antProject = null;
      if (nativeProject != null) {
        try {
          antProject = new AntProject(nativeProject.getProjectPath().getFullName());
        }
        catch (Exception e) {
          e.printStackTrace();
        }
        String enProjName = nativeProject.getDisplayName();
        enProjName = enProjName.substring(0, enProjName.lastIndexOf("."));
        nativeProject.setProperty("sys", "enhydra.project", "true");
        nativeProject.setProperty("sys", "Libraries", "Enhydra");
        nativeProject.setProperty("sys", "antlibraries", "Enhydra");
        nativeProject.setProperty("runtime", "DefaultConfiguration", "0");
        nativeProject.setProperty("runtime", "ExcludeDefaultForZero", "0");
        nativeProject.setProperty("runtime.0", "BuildTargetOnRun",
                                  "com.borland.jbuilder.build.ProjectBuilder" +
                                  "$ProjectBuildAction;rebuild");
        nativeProject.setProperty("runtime.0", "ConfigurationName", enProjName);
        nativeProject.setProperty("runtime.0", "RunnableType",
            "com.borland.jbuilder.runtime.ApplicationRunner");
        nativeProject.setProperty("runtime.0", "application.class",
                                  "com.lutris.multiServer.MultiServer");
        File outputDir = new File(antProject.getProperty(AntProject.OUTPUT_DIR));
        nativeProject.setProperty("runtime.0", "application.parameters",
                                  outputDir.getName()
                                  + "/conf/multiserver.conf");
        // set source path:
//        Url url = new Url(new File(nativeProject.getProjectPath().getFullName()
//                                    + File.separator
//                                    + antProject.getProperty(
//                                          AntProject.XMLC_GENERETED_SRC_DIR)));
        Url url = new Url(new File(antProject.getProperty(AntProject.XMLC_GENERETED_SRC_DIR)));
        doUpdateSourcePath(nativeProject, url);
//        url = new Url(new File(nativeProject.getProjectPath().getFullName()
//                               + File.separator
//                               + antProject.getProperty(
//                                     AntProject.DODS_GENERATED_SRC_DIR)));
        url = new Url(new File(antProject.getProperty(AntProject.DODS_GENERATED_SRC_DIR)));
        doUpdateSourcePath(nativeProject, url);
        nativeProject.setProperty("sys", "DefaultPath", "src");

      }
    }

    /**
     * Add needed source path to project.
     *
     * This routine checks if the given directory is already in the project
     * source path.  If not, then it is added.
     *
     * @param jbProject Project where nodes are to be added.
     * @param url Root directory for the search.
     */
    private void doUpdateSourcePath(JBProject jbProject, Url url) {
      String dir = url.getFileObject().getAbsolutePath();
      Url[] urls = jbProject.getPaths().getSourcePath();
      for (int j = 0; j < urls.length; j++) {
        String path = urls[0].getFileObject().getAbsolutePath();
        if (path.equals(dir)) {
          return;
        }
      }

      Url[] newUrls = new Url[urls.length + 1];
      int k;
      for (k = 0; k < urls.length; k++) {
        newUrls[k] = urls[k];
      }
      newUrls[k] = url;
      jbProject.getPaths().setSourcePath(newUrls);
    }

    /**
     * Method declaration
     *
     */
    private void nativeSetupProject(JDKPathSet jdk) {
        File         sourceDir = null;
        if (!nativeProject.getPaths().putClassOnFullPath(ToolBoxInfo.getEnhydraMainClass())) {

            // no enhydra library
        }
        if (!ToolBoxInfo.isEnhydra3()) {
            if (jdk != null) {
                nativeProject.getPaths().setJDKPathSet(jdk);
            }
        }
        JavaBuildPropertyGroup.SYNC_DIR.setValue(nativeProject,
                                                 String.valueOf(false));
    }

    private JDKPathSet getTargetJDK() {
        ArrayList  jdkList = PathSetManager.getJDKs();
        Iterator   jdkIterator = jdkList.iterator();
        JDKPathSet jdk = null;

        while (jdkIterator.hasNext()) {
            Object next = jdkIterator.next();

            if (next instanceof JDKPathSet) {
                jdk = (JDKPathSet) next;
                if (jdk.getName().indexOf(Constants.JDK13) > 0) {
                    break;
                }
            }
        }
        return jdk;
    }

    private void openReadme(Browser browser) {
        Node[] list = new Node[0];
        Node   node = null;

        list = nativeProject.getChildren();
        for (int i = 0; i < list.length; i++) {
            if (list[i].getDisplayName().toLowerCase().endsWith(Constants.FILE_README)) {
                node = list[i];
                break;
            }
        }
        list = new Node[0];
        try {
            browser.setActiveNode(node, true);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    abstract public String[] getSteps();

    /**
     * Class declaration
     *
     *
     * @author Paul Mahar
     */
    private class LocalWizardPage extends BasicWizardPage {
        private CodeGenPanel panel;

        /**
         * Constructor declaration
         *
         */
        public LocalWizardPage() {
            BorderLayout layout = new BorderLayout();

            layout.setHgap(5);
            layout.setVgap(5);
            this.setLayout(layout);
            if (KelpInfo.isToolBoxVersionInSynch()) {
              this.setLargeIcon(IconSet.getOtterWizard());
            }
        }

        /**
         * Method declaration
         *
         *
         * @param host
         */
        public void activated(WizardHost host) {
            host.setFinishEnabled(true);
        }

        public void checkPage() throws VetoException {
            try {
                panel.validateOptionSet();
            } catch (ValidationException e) {
                JOptionPane.showMessageDialog(panel.getTopLevelAncestor(),
                                              e.getValidationMessage(),
                                              ResUtil.format(res.getString("_0_Wizard_Error"), getDisplayName()),
                                              JOptionPane.WARNING_MESSAGE);
                throw new VetoException(e.getValidationMessage());
            }
        }

        protected void setPanel(CodeGenPanel p) {
            panel = p;
            setPageTitle(panel.getPageTitle());
            setInstructions(panel.getInstructions());
            add(panel, BorderLayout.CENTER);
        }

        protected CodeGenPanel getPanel() {
            return panel;
        }

    }
}
