package org.enhydra.kelp.ant;

import org.enhydra.kelp.ant.node.AntProject;


import java.util.ArrayList;
import java.util.HashMap;
import java.util.StringTokenizer;

import org.apache.crimson.tree.TextNode;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import java.util.regex.Matcher;
import java.util.regex.Pattern;
//import org.apache.crimson.tree.ElementNode2;
//import org.apache.crimson.tree.ElementNode;
import org.w3c.dom.Element;
import org.w3c.dom.Text;



/**
 * @author Tweety
 *
 */
public class AntFacade {

        private static final String BUILD_XML = "build.xml";

	private AntXMLUtil util = null;
        private AntProject project = null;
	private String projectRootPath = null;

	public AntFacade(AntProject project) throws Exception {
                this.project = project;
		this.projectRootPath = project.getWorkingPath();
		this.util = new AntXMLUtil(projectRootPath + "/" + BUILD_XML);
	}


	public void setPropertyLocation(String name, String location) {
		Node propertyNode = util.getProjectProperty(name);
		Node propertyValueNode = propertyNode.getAttributes().getNamedItem("location");
		propertyValueNode.setNodeValue(location);
	}

	public String getPropertyLocation(String name) {
		Node propertyNode = util.getProjectProperty(name);
		if (propertyNode == null) {
			return null;
		}
		if (propertyNode.getAttributes().getNamedItem("location") == null) {
			return null;
		}
		String result = propertyNode.getAttributes().getNamedItem("location").getNodeValue();
		return this.expandString(result);
	}



	/*
	 * Special case:
	 * if name="xmlc.package.name" then {
	 * change value of property "content.package.dir" and replace "." with "/" }
	 * */
	public void setPropertyValue(String name, String value) {
		Node propertyNode = util.getProjectProperty(name);
		Node propertyValueNode = propertyNode.getAttributes().getNamedItem("value");
		propertyValueNode.setNodeValue(value);
		if(name.equalsIgnoreCase("xmlc.package.name")) {
//			Node xmlcPackagePropertyNode = util.getProjectProperty("content.package.dir");
//			Node xmlcPackagePropertyValueNode = xmlcPackagePropertyNode.getAttributes().getNamedItem("value");
			value = value.replace('.', '/');
			Node contentPackagePropertyNode = util.getProjectProperty("content.package.dir");
		        if(contentPackagePropertyNode!=null){//WebApp do not have property "content.package.dir"
		        	Node contentPackageValueNode = contentPackagePropertyNode.getAttributes().getNamedItem("value");
					contentPackageValueNode.setNodeValue(value);
		        }
		}
	}

	public String getPropertyValue(String name) {
		Node propertyNode = util.getProjectProperty(name);
		if (propertyNode == null) {
			return null;
		}
		if (propertyNode.getAttributes().getNamedItem("value") == null) {
		    return null;
		}
		String result = propertyNode.getAttributes().getNamedItem("value").getNodeValue();
		return this.expandString(result);
	}



	/*
	 * Remove all inner include tags and add all strings as new includes
	 *
	 */
	public void replacePatternset(String id, String[] includes) {
		Node oldPatternsetNode = util.getPatternsetById(id);

		Node patternsetNode = oldPatternsetNode.cloneNode(false);
                //remove all existing filter nodes
                oldPatternsetNode.getParentNode().replaceChild(patternsetNode, oldPatternsetNode);

                Text newLineTextNode = util.document.createTextNode("\n");
		Text tabTextNode = util.document.createTextNode("    ");

		//add new includes:
		for(int i = 0; i < includes.length; i++) {

                        patternsetNode.appendChild(newLineTextNode.cloneNode(true));
                        patternsetNode.appendChild(tabTextNode.cloneNode(true));
                        patternsetNode.appendChild(tabTextNode.cloneNode(true));

                        Element newIncludeNode = util.document.createElement("include");
                        newIncludeNode.setAttribute("name", includes[i]);
			patternsetNode.appendChild(newIncludeNode);
		}
                //enter and tab for ending tag patternset
                patternsetNode.appendChild(newLineTextNode.cloneNode(true));
                patternsetNode.appendChild(tabTextNode.cloneNode(true));
        }


	public String[] getPatternset(String id) {
		Node patternsetNode = util.getPatternsetById(id);
                if(patternsetNode != null) {
                    NodeList includeChilds = patternsetNode.getChildNodes();
                    ArrayList list = new ArrayList();
                    for(int i = 0; i < includeChilds.getLength(); i++) {
                            if(includeChilds.item(i).getNodeName().equalsIgnoreCase("include"))
                                    list.add(includeChilds.item(i).getAttributes().getNamedItem("name").getNodeValue());
                    }

                    String[] s = new String[list.size()];
                    for(int i = 0; i < s.length; i++) {
                            s[i] = this.expandString(list.get(i).toString());
                    }
                    return s;
                }
                return null;
	}



	/*
	 * <filterset id="replacements">
	 * replace means delete all fiter tags and create new from HashMap
	 */
	public void replaceInputFilters(HashMap filters) {
//		Node oldInputTargetNode = util.getTargetByName("input");
		Node oldInputTargetNode = util.getFiltersetById("replacements");
		NodeList childs = oldInputTargetNode.getChildNodes();

                Node inputTargetNode = oldInputTargetNode.cloneNode(false);

                //remove all existing filter nodes
                Node first = oldInputTargetNode.getFirstChild();
                while(first != null) {
                    if(first.getNodeName().equals("filter")) {
                        first = first.getNextSibling();
                        if(first.getNodeType() == Node.TEXT_NODE)
                           first = first.getNextSibling();
                    }
                    else {
                        inputTargetNode.appendChild(first.cloneNode(true));
                        first = first.getNextSibling();
                    }
                }
                oldInputTargetNode.getParentNode().replaceChild(inputTargetNode, oldInputTargetNode);

                Text newLineTextNode = util.document.createTextNode("\n");
		Text tabTextNode = util.document.createTextNode("    ");

		//add new filter nodes
                Object[] keys = filters.keySet().toArray();
		for(int i = 0; i < filters.size(); i++) {

                        inputTargetNode.appendChild(newLineTextNode.cloneNode(true));
                        inputTargetNode.appendChild(tabTextNode.cloneNode(true));
                        inputTargetNode.appendChild(tabTextNode.cloneNode(true));

                        Element newFilterNode = util.document.createElement("filter");
                        newFilterNode.setAttribute("token", keys[i].toString());
			newFilterNode.setAttribute("value", filters.get(keys[i]).toString());
			inputTargetNode.appendChild(newFilterNode);
		}
                inputTargetNode.appendChild(newLineTextNode.cloneNode(true));
                inputTargetNode.appendChild(tabTextNode.cloneNode(true));
	}


	/*
	 * When 'value' attribut contains string that begins with '${'
	 * property named in '${...}' is found, and its 'value' is placed
	 * into HashMap
	 */
	public HashMap getInputFilters() {
//		Node inputTargetNode = util.getTargetByName("input");
				Node inputTargetNode = util.getFiltersetById("replacements");
                HashMap hash = new HashMap();
                if(inputTargetNode != null) {
                    NodeList childs = inputTargetNode.getChildNodes();

                    //find filter nodes and put them into HashMap:
                    for(int i = 0; i < childs.getLength(); i++) {
                            if(childs.item(i).getNodeName().equals("filter")) {
                                    String key = childs.item(i).getAttributes().getNamedItem("token").getNodeValue();
                                    String value = childs.item(i).getAttributes().getNamedItem("value").getNodeValue();
                                    //hash.put(key, this.expandString(value)); DACHA 28.2.2003
                                    hash.put(key, value);
                            }
                    }
                }

		return hash;
	}

	/*
	 * This method adds dependedName into depends list of target
	 * only if it not exists in the list
	 */
	public void addDependedTarget(String name, String dependedName) {
		Node targetNode = util.getTargetByName(name);
		String dependsValue = targetNode.getAttributes().getNamedItem("depends").getNodeValue();
		if(dependsValue.indexOf(dependedName) == -1) {
			String newList = this.addStringToDependensList(dependsValue, dependedName/*, name*/);
			targetNode.getAttributes().getNamedItem("depends").setNodeValue(newList);
		}
	}

	/*
	 * This method removes dependedName from depends list of the target
	 */
	public void removeDependedTarget(String name, String dependedName) {
		Node targetNode = util.getTargetByName(name);
		String dependsValue = targetNode.getAttributes().getNamedItem("depends").getNodeValue();
		if(dependsValue.indexOf(dependedName) != -1) {
			String newList = this.removeStringFromDependensList(dependsValue, dependedName);
			targetNode.getAttributes().getNamedItem("depends").setNodeValue(newList);
		}
	}

	/*
	 * @return true if dependedName is in the depends list
	 * otherwise, false
	 */
	public boolean checkDependedTarget(String name, String dependedName) {
		Node targetNode = util.getTargetByName(name);
		String dependsValue = targetNode.getAttributes().getNamedItem("depends").getNodeValue();
		if(dependsValue.indexOf(dependedName) != -1)
			return true;
		else
			return false;
	}



	private String addStringToDependensList(String dependsList, String item) {

		StringTokenizer tokenizer = new StringTokenizer(dependsList, ",");
		HashMap hash = new HashMap();

		while(tokenizer.hasMoreTokens()){
			String token = tokenizer.nextToken().trim();
			hash.put(this.getOrdinalForDependsString(token), token);
		}
		hash.put(this.getOrdinalForDependsString(item), item);

		String newList = "";
		if(hash.containsKey(new Integer(1))) newList += hash.get(new Integer(1)) + ", ";
		if(hash.containsKey(new Integer(2))) newList += hash.get(new Integer(2)) + ", ";
		if(hash.containsKey(new Integer(3))) newList += hash.get(new Integer(3)) + ", ";
		if(hash.containsKey(new Integer(4))) newList += hash.get(new Integer(4)) + ", ";

		if(newList.endsWith(", "))
			newList = newList.substring(0, newList.length() - 2);

		return newList;
	}


	/*
	 * @return index of the dependedName in list
	 *
	 * make:	xmlc,  dods,    compile, 	deploy
	 * deploy:	input, content, archive
	 */
	private Integer getOrdinalForDependsString(String item) {
		if(item.equalsIgnoreCase("xmlc") || item.equalsIgnoreCase("input"))
			return new Integer(1);
		if(item.equalsIgnoreCase("dods") || item.equalsIgnoreCase("content"))
			return new Integer(2);
		if(item.equalsIgnoreCase("compile") || item.equalsIgnoreCase("archive"))
			return new Integer(3);
		if(item.equalsIgnoreCase("deploy") || item.equalsIgnoreCase("makeWAR"))
			return new Integer(4);

		return new Integer(0);
	}


	private String removeStringFromDependensList(String dependsList, String item) {
		StringTokenizer tokenizer = new StringTokenizer(dependsList, ",");
		String newList = "";

		while(tokenizer.hasMoreTokens()) {
			String token = tokenizer.nextToken().trim();
			if(!token.equalsIgnoreCase(item))
				newList += token + ", ";
		}

		if(newList.endsWith(", "))
			newList = newList.substring(0, newList.length() - 2);

		return newList;
	}


        public boolean save() {
            return util.saveDocument();
        }

	String expandString(String str) {

		String replacement = null;
		StringBuffer sb = new StringBuffer(str);

		Pattern p = Pattern.compile("\\$\\{[^\\}]*\\}");
		Matcher m = p.matcher(sb);

		while(m.find()) {
			String propName = m.group(0).substring(2, m.group(0).length()-1);
			replacement = getProperty(propName);

			if(replacement != null) {
				propName = "${" + propName + "}";
				sb.replace(sb.indexOf(propName), sb.indexOf(propName) + propName.length(), replacement);
			}
			//else ant variable is not resolved !!!
		}

		return sb.toString();
	}


	private String getProperty(String propertyName) {

		String result = null;

		Node propertyNode = util.getProjectProperty(propertyName);
		try {
			result = propertyNode.getAttributes().getNamedItem("location").getNodeValue();
		} catch(NullPointerException e) {
			result = null;
		}


		if(result == null) {
			try {
				result = propertyNode.getAttributes().getNamedItem("value").getNodeValue();
			} catch(NullPointerException e) {
				result = null;
			}
		}

		if(result != null)
			result = this.expandString(result);

		return result;
	}

}
