package org.enhydra.kelp.ant.node;


//AntKelp imports
import org.enhydra.kelp.ant.AntFacade;
import org.enhydra.kelp.ant.node.AntNodeReader;
//Kelp imports
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterFolderNode;
import org.enhydra.kelp.common.node.OtterDocumentNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterFileNode;
import org.enhydra.kelp.common.node.OtterTextFileNode;
import org.enhydra.kelp.common.node.OtterNodeFactory;
//Toolbox imports
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.ToolBoxInfo;
// JDK
import java.io.File;
import java.util.ArrayList;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.FileWriter;
import java.io.BufferedWriter; 
import java.io.PrintWriter;
import java.util.HashMap;

/**
 * <p>Title: </p>
 * <p>Description: This class implementing OtterProject independet from IDE. It
 * use <code>build.xml</code> file for holding properties and <code>Ant tool
 * </code> for: XML compiling, DODS generator and application deploing. When
 * this class is instantiaded, must call <code>setWorkingPath()</code> where
 * <code>build.xml</code> file is placed.
 * </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Damir Milovic
 * @version 1.0
 */

public class AntProject extends OtterProject {

    //Ant Project property keys:
    public final static String  PROJECT_NAME = "project.name";
    public final static String  RESOURCES_DIR = "resources.dir";
    public final static String  SRC_DIR = "src.dir";
    public final static String  CLASSES_DIR = "classes.dir";
    public final static String  CLASS_PATH = "class.path";
    public final static String  ENHYDRA_DIR = "enhydra.dir";
    public final static String  OUTPUT_FILE_NAME = "output.file.name";	//log file
    //XMLC
    public final static String  XMLC_INVOKE = "xmlc.invoke";
    public final static String  XMLC_VERBOSE = "xmlc.verbose";
    public final static String  XMLC_PACKAGE_NAME = "xmlc.package.name";
    public final static String  XMLC_LOG_ENABLED = "xmlc.log.enabled";
    public final static String  XMLC_DEFAULT_PACKAGE = "xmlc.default.package"; //for internal use
    public final static String  XMLC_GENERETED_SRC_DIR ="xmlc.src.dir";
    public final static String  XMLC_OPTIONS_FILE = "xmlc.options.file";
    public final static String  XMLC_INCLUDES = "xmlc.includes";
    public final static String  XMLC_RESOURCES_ALL = "xmlc.resources.all";
    public final static String  XMLC_RESOURCES_SELECTED = "xmlc.resources.selected";
    public final static String  XMLC_RESOURCE_EXTENSIONS = "xmlc.resources.all";
    //Deployer
    public final static String  DEPLOY_INVOKE = "deploy.invoke";
    public final static String  DEPLOY_CONTENT_ENABLED = "deploy.content.enabled";
    public final static String  DEPLOY_INPUT_ENABLED = "deploy.input.enabled";
    public final static String  DEPLOY_WAR_ENABLED = "deploy.war.enabled";
    public final static String  DEPLOY_WAR_ENHYDRA_LIBS = "deploy.war.enhydra.libs";
    private final static String  DEPLOY_WAR_ENHYDRA_LIBS_TRUE = "enhydra.libs.true";
    private final static String  DEPLOY_WAR_ENHYDRA_LIBS_FALSE = "enhydra.libs.false";
    public final static String  INPUT_DIR = "input.dir";
    public final static String  OUTPUT_DIR = "output.dir";
    public final static String  CONTENT_DIR = "content.dir";
    public final static String  CONTENT_PACKAGE_DIR = "content.package.dir";
    public final static String  CONTENT_TYPE_EXTENSIONS = "content.types.selected";
    //DODS
    public final static String  DODS_GENERATED_SRC_DIR = "dods.src.dir";
    public final static String  DODS_DOML = "dods.doml";
    public final static String  DODS_ACTION = "dods.action";
    public final static String  DODS_TEMPLATE_SET = "dods.template.set";
    public final static String  DODS_INVOKE = "dods.invoke";


    private String workingDir = null;
    private String enhydraDir = ToolBoxInfo.getEnhydraRoot();
    private String rootPath;
    private AntNodeFactory factory = new AntNodeFactory();
    private AntFacade antFacade = null;
    private boolean webType;
    private AntNodeReader nodeReader = null;
    private String outputFileName = null;
    private boolean outputFileEnabled = false;
    private String defaultPackage = "";

    public AntProject(String rootPath) throws Exception {
        workingDir = rootPath;
        antFacade = new AntFacade(this);
        webType = true;
        nodeReader = new AntNodeReader(this);
        defaultPackage = new String(this.getProperty(PROJECT_NAME)+".presentation");
        if (antFacade.getPropertyLocation(CONTENT_DIR) == null) {
            webType = false;
        }
    }

    public OtterFileNode[] getAllInput() {
        return nodeReader.getTextFileNodes(this.getProperty(AntProject.INPUT_DIR));
    }
    public void configureRunClass() {
        /**@todo: implement this org.enhydra.kelp.common.node.OtterProject abstract method*/
    }
    public OtterFileNode[] findFileNodesByType(String[] parm1) {
        return new OtterFileNode[0];
        /**@todo: implement this org.enhydra.kelp.common.node.OtterProject abstract method*/
    }
    public void save() {
        antFacade.save();
    }
    public OtterDocumentNode[] getAllDocuments() {
        return nodeReader.getDocumentNodes(this.getProperty(AntProject.RESOURCES_DIR));
    }
    /**
     * AntXMLCTool call this method when selection is changed
     * @param docNodes
     */
    public void setAllDocuments(OtterDocumentNode[] docNodes){

        ArrayList selectedList = new ArrayList();
        for(int i=0;i<docNodes.length;i++){
            if(docNodes[i].isSelected())
                selectedList.add(docNodes[i]);
        }
        Object[] objs = selectedList.toArray();
        OtterDocumentNode[] selectedDocNodes = new OtterDocumentNode[objs.length];
        for(int i=0;i<objs.length;i++){
            selectedDocNodes[i] = (OtterDocumentNode)objs[i];
        }

        if(selectedDocNodes.length == docNodes.length){    //All documents are selected
            this.setProperty(this.XMLC_INCLUDES,this.XMLC_RESOURCES_ALL); //Set all for XML compilation
            antFacade.replacePatternset(this.XMLC_RESOURCES_SELECTED, new String[0]); //clear previous selected docs
        }else{
            String resourceDir = relativeToAbsolute(getProperty(this.RESOURCES_DIR));
            this.setProperty(this.XMLC_INCLUDES,this.XMLC_RESOURCES_SELECTED);//Set selected for XML compilation
            String[] selectedPaths = new String[selectedDocNodes.length];
            for(int i=0;i<selectedDocNodes.length;i++){
                selectedPaths[i] = selectedDocNodes[i].getFilePath();
                selectedPaths[i] = absoluteToRelative(resourceDir, selectedPaths[i]);//make relative to resources dir
            }
            antFacade.replacePatternset(this.XMLC_RESOURCES_SELECTED, selectedPaths);
        }
    }
    /**
     * Get selected documents for XMLC from build.xml file
     * @return selected documents
     */
    public OtterDocumentNode[] getSelectedDocuments() {
        String[] docPaths = antFacade.getPatternset(this.XMLC_RESOURCES_SELECTED);
        OtterDocumentNode[] docNodes = null;
        if(docPaths!=null){
            docNodes = new OtterDocumentNode[docPaths.length];
            for(int i=0; i<docPaths.length; i++){
                docNodes[i] = factory.createDocumentNode(this, resourceRelativeToAbsolute(docPaths[i]));
            }
        }else{
            docNodes = new OtterDocumentNode[0];
        }
        return docNodes;
    }
    public String getRootPath() {
        return getWorkingPath(); //FIXME, for now it is the same
    }

    public Object getNativeNode() {
        return null;
        /**@todo: implement this org.enhydra.kelp.common.node.OtterProject abstract method*/
    }
    public String getProperty(String property) {
        String s = null;
        if (property.equalsIgnoreCase(NAME_WORKING_DIR)){
            s = workingDir;
        } else if(property.equalsIgnoreCase(ENHYDRA_DIR)){
            s = enhydraDir;
        } else if(property.equalsIgnoreCase(RESOURCES_DIR)){
            s = relativeToAbsolute(antFacade.getPropertyValue(RESOURCES_DIR));
        } else if(property.equalsIgnoreCase(DODS_DOML)){
            s = relativeToAbsolute(antFacade.getPropertyLocation(DODS_DOML));
        } else if (property.equalsIgnoreCase(AntProject.XMLC_INVOKE)) {
            boolean checkInvoke = antFacade.checkDependedTarget("make","xmlc");
            if(checkInvoke)
                s = "true";
            else
                s = "false";
        } else if (property.equalsIgnoreCase(AntProject.DODS_INVOKE)) {
            boolean checkInvoke = antFacade.checkDependedTarget("make","dods");
            if(checkInvoke)
                s = "true";
            else
                s = "false";
        } else if (property.equalsIgnoreCase(AntProject.DODS_TEMPLATE_SET)) {
            s = antFacade.getPropertyValue(property);
        } else if (property.equalsIgnoreCase(AntProject.DODS_ACTION)) {
            s = antFacade.getPropertyValue(property);
        } else if (property.equalsIgnoreCase(AntProject.DEPLOY_CONTENT_ENABLED)) {
            boolean checkInvoke = antFacade.checkDependedTarget("deploy","content");
            if(checkInvoke)
                s = "true";
            else
                s = "false";
        } else if (property.equalsIgnoreCase(AntProject.DEPLOY_INPUT_ENABLED)) {
            boolean checkInvoke = antFacade.checkDependedTarget("deploy","input");
            if(checkInvoke)
                s = "true";
            else
                s = "false";
        } else if (property.equalsIgnoreCase(AntProject.DEPLOY_WAR_ENABLED)) {
            boolean checkInvoke = antFacade.checkDependedTarget("deploy","makeWAR");
            if(checkInvoke)
                s = "true";
            else
                s = "false";
        } else if (property.equalsIgnoreCase(AntProject.DEPLOY_INVOKE)) {
            boolean checkInvoke = antFacade.checkDependedTarget("make","deploy");
            if(checkInvoke)
                s = "true";
            else
                s = "false";
        } else if (property.equalsIgnoreCase(AntProject.DEPLOY_WAR_ENHYDRA_LIBS)) {
            String value = antFacade.getPropertyValue(property);
            if(value.equalsIgnoreCase(this.DEPLOY_WAR_ENHYDRA_LIBS_TRUE))
                s = "true";
            else
                s = "false";
        } else if (property.equalsIgnoreCase(AntProject.OUTPUT_FILE_NAME)) {
            s = outputFileName;
        } else if (property.equalsIgnoreCase(AntProject.XMLC_DEFAULT_PACKAGE)) {
            s = defaultPackage;
        } else if (s == null) {
            if ((s = antFacade.getPropertyValue(property)) == null)
                s = relativeToAbsolute(antFacade.getPropertyLocation(property));
        }
        return s;

    }

    public void setProperty(String property, String value) {
        if(property.equalsIgnoreCase(DEPLOY_INVOKE)){
            if (value.equalsIgnoreCase("true")) {
                antFacade.addDependedTarget("make", "deploy");
            }
            else {
                antFacade.removeDependedTarget("make", "deploy");
            }
        } else if(property.equalsIgnoreCase(OUTPUT_DIR)) {
            String relativePath = absoluteToRelative(this.getWorkingPath(),value);
            antFacade.setPropertyLocation(property, relativePath);
        } else if(property.equalsIgnoreCase(DODS_DOML)) {
            String relativePath = absoluteToRelative(this.getWorkingPath(),value);
            antFacade.setPropertyLocation(property, relativePath);
        } else if(property.equalsIgnoreCase(DODS_ACTION)) {
            antFacade.setPropertyValue(property, value);
        } else if(property.equalsIgnoreCase(DODS_TEMPLATE_SET)) {
            antFacade.setPropertyValue(property, value);
        } else if(property.equalsIgnoreCase(INPUT_DIR)) {
            String relativePath = absoluteToRelative(this.getWorkingPath(),value);
            antFacade.setPropertyLocation(property, relativePath);
        } else if(property.equalsIgnoreCase(DEPLOY_INPUT_ENABLED)){
            if (value.equalsIgnoreCase("true")) {
                antFacade.addDependedTarget("deploy", "input");
            }
            else {
                antFacade.removeDependedTarget("deploy", "input");
            }
        } else if(property.equalsIgnoreCase(DEPLOY_CONTENT_ENABLED)){
            if (value.equalsIgnoreCase("true")) {
                antFacade.addDependedTarget("deploy", "content");
            }
            else {
                antFacade.removeDependedTarget("deploy", "content");
            }
        } else if(property.equalsIgnoreCase(DEPLOY_WAR_ENABLED)){
            if (value.equalsIgnoreCase("true")) {
                antFacade.addDependedTarget("deploy", "makeWAR");
            }
            else {
                antFacade.removeDependedTarget("deploy", "makeWAR");
            }
        } else if(property.equalsIgnoreCase(DEPLOY_WAR_ENHYDRA_LIBS)){
            if (value.equalsIgnoreCase("true"))
                antFacade.setPropertyValue(property,DEPLOY_WAR_ENHYDRA_LIBS_TRUE);
            else
                antFacade.setPropertyValue(property,DEPLOY_WAR_ENHYDRA_LIBS_FALSE);

        } else if(property.equalsIgnoreCase(XMLC_OPTIONS_FILE)){
            antFacade.setPropertyValue(property, value);
        } else if(property.equalsIgnoreCase(XMLC_INVOKE)){
            if (value.equalsIgnoreCase("true")) {
                antFacade.addDependedTarget("make", "xmlc");
            }
            else {
                antFacade.removeDependedTarget("make", "xmlc");
            }
        }else if(property.equalsIgnoreCase(DODS_INVOKE)){
            if (value.equalsIgnoreCase("true")) {
                antFacade.addDependedTarget("make", "dods");
            }
            else {
                antFacade.removeDependedTarget("make", "dods");
            }
        } else if(property.equalsIgnoreCase(XMLC_VERBOSE)) {
            antFacade.setPropertyValue(property, value);
        } else if(property.equalsIgnoreCase(XMLC_PACKAGE_NAME)) {
            antFacade.setPropertyValue(property, value);
        } else if(property.equalsIgnoreCase(OUTPUT_FILE_NAME)) {
            outputFileName = value;
        }else if(property.equalsIgnoreCase(this.XMLC_INCLUDES)) {
            antFacade.setPropertyValue(property,value);
        }
        //antFacade.setProperty(property,value);
    }

    public void setProperty(String property, int value) {
        setProperty(property,Integer.toString(value));
    }

    public void setNativeNode(Object parm1) {
        ;//Don't need NativeNode any more
    }
    public String[] getSourcePathArray() {
        //FIXME include /src and genereted sources dirs (xmlc & dods)
        return null;
    }

    public String getSourcePath() {
        return getProperty(AntProject.SRC_DIR);
    }
    public boolean isOpenBuild() {
        return true;
        /**@todo: implement this org.enhydra.kelp.common.node.OtterProject abstract method*/
    }
    public String getClassPath() {
        //return rootPath+"/classes";
        return getProperty(AntProject.CLASS_PATH);//FIXME TODO MAYBE implement method getClassPath() in antFacade
    }
    public boolean isDefaultProject() {
        return false;
    }
    public OtterTextFileNode[] getAllDeploymentDescs() {
        return new OtterTextFileNode[0];  //FIXME

    }
    public Object getNativeProject() {
        return null;  //No native project for now
    }
    public OtterNodeFactory getNodeFactory() {
        return factory;

    }
    public OtterNode getParent() {
        return null;
    }

    public String getWorkingPath() {
        return getProperty(NAME_WORKING_DIR);  //FIXME for now workingPath = rootPath
    }

    public void setWorkingPath(String path) {
        setProperty(NAME_WORKING_DIR,path);
    }
    public OtterProject getProject() {
        return this;
    }
    public String getClassOutputPath() {
        return getProperty(AntProject.CLASSES_DIR);
    }
    public void setClassOutputPath(String p) {
        setProperty(AntProject.CLASSES_DIR,p);
    }
    public String getCodeGenDefaultDestination() {
        return rootPath+"/xmlc_generated_source"; //FIXME read from build.xml

    }
    public void setSourcePathArray(String[] parm1) {
        /**@todo: implement this org.enhydra.kelp.common.node.OtterProject abstract method*/
    }

    public OtterTextFileNode getFirstPolicy() {
        return null;
        /**@todo: implement this org.enhydra.kelp.common.node.OtterProject abstract method*/
    }

    /**
     * Converts relative paths from "foo" to "./foo" to comply with
     * PathHandle.expandRelativePath()
     */
    private String relativePathAdapter(String path) {
        String newPath = null;
        File file = new File(path);
        if (!file.isAbsolute())
            newPath = "./" + path;
        else
            newPath = path;
        return newPath;
    }
    /**
     * Make absolute path from path relative to working dir.
     * @param relativePath Relative path
     * @return Absolute path
     */
    private String relativeToAbsolute(String relativePath){
        if(relativePath!=null){
            PathHandle workDirHandle = PathHandle.createPathHandle(workingDir);
            PathHandle absoluteHandle = workDirHandle.expandRelativePath(relativePathAdapter(relativePath));
            return absoluteHandle.getPath();
        }else
            return null;
    }

    private String resourceRelativeToAbsolute(String relativePath){
        if(relativePath!=null){
            String resourceDir = relativeToAbsolute(getProperty(this.RESOURCES_DIR));
            PathHandle resourseHandle = PathHandle.createPathHandle(resourceDir);
            PathHandle absoluteHandle = resourseHandle.expandRelativePath(relativePathAdapter(relativePath));
            return absoluteHandle.getPath();
        }else
            return null;
    }
    
    private String absoluteToRelative(String homePath,String inPath){
        PathHandle homePathHandle = PathHandle.createPathHandle(homePath);
        PathHandle inPathHandle = PathHandle.createPathHandle(inPath);
        String relativePath = homePathHandle.getRelativePath(inPathHandle);
        //cut "./"
        if(relativePath.startsWith("./"))
            relativePath = relativePath.substring(2);
        return relativePath;
    }
    /**
     * Read options from file
     * @return lines from xmlc options file
     */
    public String[] getXMLCOptionsFileParams() {
        String optFilePath = getProperty(AntProject.XMLC_OPTIONS_FILE);
        optFilePath = relativePathAdapter(optFilePath);
        PathHandle resourcesHandle =
                PathHandle.createPathHandle(getProperty(AntProject.RESOURCES_DIR));
        PathHandle optFileHandle = resourcesHandle.expandRelativePath(optFilePath);
        FileReader optFileReader = null;
        BufferedReader br = null;
        ArrayList optionsList = new ArrayList();
        try {
            optFileReader = new FileReader(optFileHandle.getFile());
            br = new BufferedReader(optFileReader, 4096);
            String optLine = br.readLine();
            while(optLine != null) {
                if (!optLine.equals(""))
                    optionsList.add(optLine);
                optLine = br.readLine();
            }
            br.close();
        }
        catch (IOException  ex) {
            ex.printStackTrace();
        }
        Object[] objs = optionsList.toArray();
        String[] options = new String[objs.length];
        for (int i = 0; i < objs.length; i++) {
            options[i] = (String)objs[i];
        }

        return options;
    }

    /**
     * Write options to file
     * @param options XMLC options
     */
    public void setXMLCOptionsFileParams(String[] options) {
        String optFilePath = getProperty(AntProject.XMLC_OPTIONS_FILE);
        optFilePath = relativePathAdapter(optFilePath);
        PathHandle resourcesHandle =
                PathHandle.createPathHandle(getProperty(AntProject.RESOURCES_DIR));
        PathHandle optFileHandle = resourcesHandle.expandRelativePath(optFilePath);
        FileWriter fw;
        try {
            fw = new FileWriter(optFileHandle.getFile());
            } catch(IOException e) {return;}
            BufferedWriter bw = new BufferedWriter(fw, 4096);
            PrintWriter prw = new PrintWriter(bw, false /* auto flush */);
            for (int i = 0; i < options.length; i++) {
                prw.println(options[i]);
            }
            prw.println();
            prw.flush();
            prw.close();
    }

    /**
     *
     * @return true if application type is web, otherwise it is enhydra app
     */
    public boolean isWebType() {
        return webType;
    }

    public String[] getExtensions(String patternsetId) {
        String[] extensions = antFacade.getPatternset(patternsetId);
        for (int i = 0; i < extensions.length; i++) {
            extensions[i] = extensions[i].substring(extensions[i].indexOf("**/*.")+5);
            extensions[i] = extensions[i].trim();
        }
        return extensions;
    }

    public void setExtensions(String patternsetId, String[] extensions) {
        String[] expandedExts = new String[extensions.length];
        for (int i = 0; i < extensions.length; i++) {
            expandedExts[i] = "**/*." + extensions[i];
        }
        antFacade.replacePatternset(patternsetId, expandedExts);
    }


    public HashMap getReplacements() {
        return antFacade.getInputFilters();
    }

    public void setReplacements(HashMap replacements) {
        antFacade.replaceInputFilters(replacements);
    }

    public boolean isOutputFileEnabled() {
        return outputFileEnabled;
    }

    public void setOutputFileEnabled(boolean enable) {
        outputFileEnabled = enable;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getOutputFilename() {
        String out = null;

        out = getProperty(OUTPUT_FILE_NAME);
        return out;
    }

    /**
     * Method declaration
     *
     *
     * @param n
     */
    public void setOutputFilename(String n) {
        setProperty(OUTPUT_FILE_NAME, n);
    }
}

