package org.enhydra.kelp.ant.xmlc;

//AddInCore
//import org.enhydra.kelp.common.xmlc.CoreXMLCTool;
//import org.enhydra.kelp.common.xmlc.XMLCInnerPanel;
import org.enhydra.kelp.common.AbstractTool;
import org.enhydra.kelp.common.event.SelectionPanelEvent;
import org.enhydra.kelp.common.event.SelectionPanelListener;
import org.enhydra.kelp.common.event.WriteListener;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterFileNode;
import org.enhydra.kelp.common.swing.AddinInnerPanel;
import org.enhydra.kelp.common.properties.XMLCProjectPropertyPanel;

//AddInAnt
import org.enhydra.kelp.ant.node.AntProject;
import org.enhydra.kelp.ant.swing.HTMLViewer;


//ToolBox
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.InnerPanel;
import org.enhydra.tool.common.SwingUtil;
import org.enhydra.tool.common.ButtonPanel;
import org.enhydra.tool.common.event.FirstFocusEvent;
import org.enhydra.tool.common.event.FirstFocusListener;

// Standard imports
import java.io.File;
import java.io.IOException;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.event.ActionListener;
import java.lang.ref.WeakReference;
import java.net.URL;
import java.util.ResourceBundle;

import javax.swing.JDialog;
import javax.swing.JEditorPane;

import org.enhydra.kelp.common.node.OtterDocumentNode;
import org.enhydra.kelp.ant.node.AntDocumentNode;
/**
 * <p>Title: </p>
 * <p>Description: XMLC tool use AntProject for read,write properties and
 * AntXMLCBuilder for running XML Compiler. </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Damir Milovic
 * @version 1.0
 */

public class AntXMLCTool extends AbstractTool
                         implements SelectionPanelListener, FirstFocusListener {

    static ResourceBundle            addinRes =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");
    private AntXMLCInnerPanel           innerPanel = null;
    private AntXMLCButtonPanel          buttonPanel = null;
    private WeakReference            compileRef = null;
    private static AntProject project = null;

  public AntXMLCTool() {
    super();
    innerPanel = new AntXMLCInnerPanel();
    buttonPanel = new AntXMLCButtonPanel();
    innerPanel.addFirstFocusListener(this);
    innerPanel.getSelectionPanel().addSelectionPanelListener(this);
  }
  public static void main(String[] args) {
        SwingUtil.setLookAndFeelToSystem();
        AntXMLCTool tool = null;

        if(args.length > 0){
          // check path
            File root = new File(args[0]);
            if(!root.exists()){
              System.err.println("ERROR: directory'"+args[0]+"' don't exist!");
              return;
            }
            try {
                project = new AntProject(args[0]);
            }
            catch (Exception ex) {
                ex.printStackTrace();
                return;
            }

        }else{
            return;
          //FIXME TODO get current directory
        }
        tool = new AntXMLCTool();
        tool.setProject(project); //DACHA
        tool.showDialog(new Frame());
        //tool.showDialog(null);
  }
  /**
   * Override
   */
  public void build() {
    ((AntXMLCInnerPanel)getInnerPanel()).selectOutputTab();
    ((AntXMLCInnerPanel)getInnerPanel()).save();
    ((AntXMLCInnerPanel)getInnerPanel()).clearOutput();
    buildImpl();
  }

  private void buildImpl() {
    AntXMLCBuilder builder = new AntXMLCBuilder(((AntXMLCInnerPanel)getInnerPanel()).getWriteListeners());
    builder.setProject(getProject());
    builder.build();
  }

    public static final String getDefaultTitle() {
        return "XML Compiler";
    }

    // implements AbstractTool
    public String getTitle() {
        return AntXMLCTool.getDefaultTitle();
    }

    // implements AbstractTool
    public InnerPanel getInnerPanel() {
        return innerPanel;
    }

    // implements AbstractTool
    public ButtonPanel getButtonPanel() {
        return buttonPanel;
    }

    public ActionListener createButtonListener() {
        return (new AntXMLCButtonListener(this));
    }

    // overrides AbstractTool
    public void setProject(OtterProject project) {
        super.setProject(project);
        setNodes(project.getAllDocuments());
    }

    // overrides AbstractTool
    public void clearAll() {
        super.clearAll();
       innerPanel.removeFirstFocusListener(this);
        if (compileRef != null) {
            compileRef.clear();
        }
        innerPanel = null;
        buttonPanel = null;
        compileRef = null;
    }

    // implements FirstFocusListener
    public void onFirstFocus(FirstFocusEvent event) {
      refreshCompile();
    }
    // implements SelectionPanelListener
    public void onSelection(SelectionPanelEvent event) {
        boolean enable = event.getSelected().size() > 0;
        //First set to project changed documents
        OtterFileNode[] fileNodes = new OtterFileNode[0];
        fileNodes =  innerPanel.getNodes();
        OtterDocumentNode[] docNodes = new OtterDocumentNode[fileNodes.length];
        for(int i=0; i<fileNodes.length;i++){
            docNodes[i] = (OtterDocumentNode) fileNodes[i];
        }
        project.setAllDocuments(docNodes);
        //refrsh Compile button
        refreshCompile(enable);
    }
    
    public void showHelp(Dialog parent){
    	URL helpURL = this.getClass().getClassLoader().getResource("using_ant_kelp.html");
		HTMLViewer.createAndShow(parent, "Kelp Help", helpURL);

    }
    //
    // PROTECTED
    //
    // implements org.enhydra.kelp.common.tool.AbstractTool

    /**
     * Get source files that can be selected for compilation.
     */
    protected OtterFileNode[] getNodes() {
        return innerPanel.getNodes();
    }

    /**
     * Set the source files that can be selected for compilation
     */
    protected void setNodes(OtterFileNode[] n) {
        innerPanel.setNodes(n);
        //refreshCompile();
    }

    protected String[] getSelectText() {
        String[] s = {
            "Documents Available", "Documents Selected"
        };
        return s;
    }

    // implements org.enhydra.kelp.common.tool.AbstractTool
    protected String getBuildText() {
        return addinRes.getString("buttonMake_Text");
    }

    // implements org.enhydra.kelp.common.tool.AbstractTool
    protected String getProgressTitle() {
        return addinRes.getString("XMLC_Progress");
    }

    // implements org.enhydra.kelp.common.tool.AbstractTool
    protected AddinInnerPanel getPropertyPanel() {
        return innerPanel;
    }

    private Component getCompile() {
        Component comp = null;

        if (compileRef == null) {
            comp = ButtonPanel.findButton(getInnerPanel().getTopLevelAncestor(),
                                       AntXMLCButtonPanel.COMMAND_COMPILE);
            if (comp != null) {
                compileRef = new WeakReference(comp);
            }
        } else {
            comp = (Component) compileRef.get();
        }
        return comp;
    }
    //Called when user change selection
    private void refreshCompile() {

         boolean         enable = false;
        OtterFileNode[] nodes = new OtterFileNode[0];
        nodes = innerPanel.getNodes();
        if(nodes != null){

            for (int i = 0; i < nodes.length; i++) {
                if (nodes[i].isSelected()) {
                    enable = true;
                    break;
                }
            }
        }
        refreshCompile(enable);
    }

    private void refreshCompile(boolean enable) {
        Component compileButton = getCompile();

        if (compileButton != null) {
            compileButton.setEnabled(enable);
        }
    }

}