/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.deployer;

// Toolbox imports
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.FileUtil;

// Kelp imports
import org.enhydra.kelp.common.AbstractEchoBuilder;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.event.WriteListener;
import org.enhydra.kelp.common.map.Mapper;
import org.enhydra.kelp.common.node.OtterImageFileNode;
import org.enhydra.kelp.common.node.OtterTextFileNode;
import org.enhydra.kelp.common.node.OtterDocumentNode;
import org.enhydra.kelp.common.node.OtterFileNode;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterProject;

//
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.ResourceBundle;

//
public class ContentBuilder extends AbstractEchoBuilder {
    static ResourceBundle res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private Mapper        mapper = null;
    private PathHandle    resourcePath = null;
    private String        copyingTo = res.getString("Copying_to");

    public ContentBuilder(WriteListener listener) {
        super(listener);
    }

    protected void buildImpl() {
        makeStaticDocuments();
        makeNonDocumentContent();
        makeDeploymentDesc();
    }

    public void setProject(OtterProject p) {
        super.setProject(p);
        mapper = new Mapper(getProject());
        resourcePath =
            PathHandle.createPathHandle(getProject().getDeployResourcePath());
    }

    private void makeStaticDocuments() {

        // Find all images in project and copy using
        // the mapping table.
        OtterDocumentNode[] docs = getProject().getAllDocuments();
        File                source = null;
        File                dest = null;
        String              destPath = new String();

        for (int i = 0; i < docs.length; i++) {
            if (docs[i].isStatic()) {
                destPath = getResourceDestPath(docs[i]);
                dest = new File(destPath);
                source = new File(docs[i].getFilePath());
                try {
                    FileUtil.copy(source, dest);
                    echo(copyingTo + destPath);
                } catch (ToolException e) {
                    echo(e);
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * Method declaration
     *
     */
    private void makeNonDocumentContent() {

        // Find all images in project and copy using
        // the mapping table.
        String[]        types = new String[0];
        OtterFileNode[] content = new OtterFileNode[0];
        PathHandle      cursorPath = null;

        types = getProject().getNonDocumentContentTypes();
        content = getProject().findFileNodesByType(types);
        File   source = null;
        File   dest = null;
        String destPath = new String();

        for (int i = 0; i < content.length; i++) {
            cursorPath =
                PathHandle.createPathHandle(content[i].getFilePath());
            if (resourcePath.parentOf(cursorPath)) {
                destPath = getResourceDestPath(content[i]);
                dest = new File(destPath);
                source = cursorPath.getFile();
                try {
                    FileUtil.copy(source, dest);
                    echo(copyingTo + destPath);
                } catch (ToolException e) {
                    echo(e);
                    e.printStackTrace();
                }
            }
        }
    }

    private void makeDeploymentDesc() {

        // Find all images in project and copy using
        // the mapping table.
        OtterTextFileNode[] descs = getProject().getAllDeploymentDescs();
        File                source = null;
        File                dest = null;
        String              sourcePath = null;
        String              rawFile = null;
        StringBuffer        outPath = new StringBuffer();

        for (int i = 0; i < descs.length; i++) {
            source = new File(descs[i].getFilePath());
            sourcePath = getProject().getSourcePathOf(descs[i]);
            rawFile = (new File(descs[i].getFilePath())).getAbsolutePath();
            outPath.append(PathUtil.getDeployContentPath(getProject()));
            outPath.append(rawFile.substring(sourcePath.length()));
            dest = new File(outPath.toString());
            try {
                FileUtil.copy(source, dest);
                echo(copyingTo + PathHandle.createPathString(dest));
            } catch (ToolException e) {
                echo(e);
                e.printStackTrace();
            }
        }
    }

    /**
     * Method declaration
     *
     * @param resource
     *
     * @return
     */
    private String getResourceDestPath(OtterFileNode resource) {
        StringBuffer destPath = new StringBuffer();
        PathHandle   sourceFilePath = null;
        int          type = OtterProject.TYPE_WEBAPP;

        type = getProject().getDeployType();
        sourceFilePath = PathHandle.createPathHandle(resource.getFilePath());
        if (type == OtterProject.TYPE_EN3APP) {
            destPath.append(mapper.getMappedOutputPath(resource));
        } else {
            String suffix = new String();

            destPath.append(PathUtil.getDeployContentPath(getProject()));
            suffix =
                sourceFilePath.getPath().substring(resourcePath.getPath().length());
            if ((suffix.length() == 0) || (suffix.charAt(0) != '/')) {
                destPath.append('/');
            }
            destPath.append(suffix);
        }
        return PathHandle.createPathString(destPath.toString());
    }

}
