/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the license.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.dods;

// Kelp imports
import org.enhydra.kelp.common.AbstractEchoBuilder;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.event.WriteListener;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.node.OtterFileNode;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterNodeFactory;

// ToolBox imports
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.Template;
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.configure.ConfigTool;
import org.enhydra.tool.common.event.ProgressListener;

// Standard imports
import java.awt.Component;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.ResourceBundle;

import java.io.*;
import javax.swing.*;
import org.enhydra.dods.xslt.*;
import org.enhydra.kelp.common.AbstractEchoBuilder;

//
public class DodsBuilder extends AbstractEchoBuilder {

    static ResourceBundle       res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private File[]              builtTemplates = new File[0];
    private File[]              builtPassthrough = new File[0];
    private OtterNode[] nodes = null;
    private Component           owner = null;

    public DodsBuilder(WriteListener listener) {
        super(listener);
    }

    protected void buildImpl() throws ToolException {

    Exception ex = null;

    try {

      String as[] = new String[5];
      if(System.getProperty("os.name").toLowerCase().startsWith("win"))
        as[0] = this.getEnhydraPath()+File.separator+"bin"+File.separator+"dods.bat";
      else
        as[0] = this.getEnhydraPath()+File.separator+"bin"+File.separator+"dods";
      as[1] = getProject().getDomlFilePath();
      as[2] = getProject().getDodsRootPath();
      as[3] = getProject().getDodsParameters();
      as[4] = getProject().getExstensionParameter();

      Process process = Runtime.getRuntime().exec(as);

      java.io.InputStream inputstream = process.getInputStream();
      BufferedReader bufferedreader = new BufferedReader(new InputStreamReader(inputstream));
      java.io.InputStream inputstream1 = process.getErrorStream();
      BufferedReader bufferedreader1 = new BufferedReader(new InputStreamReader(inputstream1));

      String str1="";
      String str2="";
      (new ErrorReader(bufferedreader1,this)).start();

      int i = 1;
      while((str1 = bufferedreader.readLine()) != null) {
        if (isFresh()) {
          echo(str1);
          if(i<90) {
            refreshProgress(i, "Generating Dods files");
          }
          i=i+2;
        } else {
            process.destroy();
            refreshProgress(100, "");
            echo("Process has been terminated");
            return;
        }
      }

      process.waitFor();
      int k = process.exitValue();

	 String dodsRootPath=getProject().getDodsRootPath();
	 if(dodsRootPath.endsWith("/"))
	 	dodsRootPath=dodsRootPath.substring(0,dodsRootPath.length()-1);

    String[] sourcePath = getProject().getSourcePathArray();
    boolean isFind = false;
    for(int j=0;j<sourcePath.length;j++) {

      if(dodsRootPath.equalsIgnoreCase(sourcePath[j]))
        isFind=true;
    }
    if(!isFind) {
      String[] newSourcePath = new String[sourcePath.length+1];
      for(int j=0;j<sourcePath.length;j++) {
        newSourcePath[j] = sourcePath[j];
        }
      newSourcePath[sourcePath.length] = getProject().getDodsRootPath();
     getProject().setSourcePathArray(newSourcePath);
    }



    } catch(Exception e) {
      ex = e;
    }

  }

    public Component getOwner() {
        return owner;
    }

    private String getEnhydraPath() {
      return getProject().getEnhydraPath();
    }

    private String getProjectJavaPath() {
      return getProject().getProjectJavaPath();
    }

}
