/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the license.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.dods;

// Kelp imports
import org.enhydra.kelp.common.AbstractEchoGenerator;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.event.WriteListener;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.node.OtterFileNode;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterNodeFactory;

// ToolBox imports
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.Template;
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.configure.ConfigTool;
import org.enhydra.tool.common.event.ProgressListener;
//import org.enhydra.tool.common.ProgressMeter;

// Standard imports
import java.awt.Component;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.ResourceBundle;

import java.io.*;
import javax.swing.*;
import org.enhydra.dods.xslt.*;
import org.enhydra.kelp.common.AbstractEchoBuilder;

//
public class Generator extends AbstractEchoBuilder {

    static ResourceBundle       res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private Component           owner = null;
    private boolean       doneDialog = false;
    private CoreDodsTool coreDodsTool = null;

    public Generator(WriteListener listener) {
        super(listener);
    }

    protected void buildImpl() throws ToolException {

    Exception ex = null;

      if(getProject().getDomlFilePath().equals("")) {
          refreshProgress(100, "");
          JOptionPane.showMessageDialog(getOwner(), "A filed, denoted by 'Doml file', has to be set.");
           return;
      }

   try {
            refreshProgress(1, res.getString("Dods_starting"));
            echo(res.getString("Dods_starting"));
            if (isFresh()) {
                buildDods();
            }
            refreshProgress(100, "");
            echo("--- Dods Generator finished --- ");

        } catch (Exception e) {
            ex = e;
            echo(ex);
        }
  }

    public void setOwner(Component c) {
        owner = c;
    }

    public Component getOwner() {
        return owner;
    }

    private String getEnhydraPath() {
      return getProject().getEnhydraPath();
    }

    private String getProjectJavaPath() {
      return getProject().getProjectJavaPath();
    }

    protected boolean isDoneDialog() {
        boolean canDone = doneDialog;

        if (getOwner() == null) {
            canDone = false;
        }
        return canDone;
    }

    protected void setDoneDialog(boolean d) {
        doneDialog = d;
    }

    private void echoWrite(String str) {
      echo(str);
    }

    public void buildDods() {

        DodsBuilder builder = null;

        builder = new DodsBuilder(getWriteListener());

        builder.addProgressListener(getProgressListeners()[0]);

        invoke(builder);
    }

    private void invoke(AbstractEchoBuilder builder) {

        ProgressListener[] pl = getProgressListeners();

        if (pl != null) {
            for (int i = 0; i < pl.length; i++) {
                builder.addProgressListener(pl[i]);
            }
        }
        builder.setProject(getProject());
        builder.setEcho(isEcho());
        builder.buildInCurrentThread();
        if (pl != null) {
            for (int i = 0; i < pl.length; i++) {
                builder.removeProgressListener(pl[i]);
            }
        }
    }

  }
