
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive.wizard;

// ToolBox imports
import org.enhydra.tool.archive.ArchiveException;
import org.enhydra.tool.archive.JarPlan;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.wizard.TBWizardPanel;

// Standard imports
import javax.swing.JPanel;
import java.io.File;
import java.lang.ref.WeakReference;
import java.util.ResourceBundle;

/**
 * This class defines the type of panel you can add to an
 * OpenTool's BasicWizardPage or an CodeGenPage. Do to limitations
 * in the JBuilder UI designer, this class and some of it's methods
 * are not left as abstract.
 */
public class ArchivePanel extends TBWizardPanel {
    private WeakReference wizardRef = null;

    /**
     * Create an CodeGenPanel for use in an OpenTool BasicWizardPage
     * or an CodeGenPage.
     */
    public ArchivePanel() {
      super();
    }

    /**
     * Get the title to use on the current page. Override this
     * to set a custom title.
     *
     * @return
     * A string to place at the top of a CodeGen wizard panel.
     */
    public String getPageTitle() {
        return "ARCHIVE PAGE TITLE";
    }

    /**
     * Populate the Swing control values from the option array.
     * Not abstract due to limitation of JBuilder designer.
     *
     * @exception GeneratorException
     * Thrown if not child class did not override.
     */
    public void readPlan(JarPlan plan) throws ArchiveException {
        ArchiveException e =
            new ArchiveException("ArchivePanel::readPlan");    // nores

        e.printStackTrace();
        throw e;
    }

    /**
     * Write values from the swing controls into the option array.
     * Not abstract due to limitation of JBuilder designer.
     *
     * @exception GeneratorException
     * Thrown if not child class did not override.
     */
    public void writePlan(JarPlan plan) throws ArchiveException {
        ArchiveException e =
            new ArchiveException("ArchivePanel::writePlan");    // nores

        e.printStackTrace();
        throw e;
    }

    /**
     * Validate values in the swing controls.
     * Not abstract due to limitation of JBuilder designer.
     *
     * @exception ValidationException
     * Thrown if not child class did not override.
     */
    public void validatePlan() throws ArchiveException {
        ArchiveException e =
            new ArchiveException("ArchivePanel::validatePlan");    // nores

        e.printStackTrace();
        throw e;
    }

    protected String getWorkingRoot() {
      String root = null;
      if (getWizard() == null) {
        root = System.getProperty("user.dir");
      } else {
         root = getWizard().getWorkingRoot();
      }
      return root;
    }

    protected void setWorkingRoot(String root) {
      if (getWizard() != null) {
         getWizard().setWorkingRoot(root);
      }
    }

    protected void preparePanel() {}

    protected ArchiveWizard getWizard() {
        ArchiveWizard wizard = null;

        if (wizardRef != null) {
            wizard = (ArchiveWizard) wizardRef.get();
        }
        return wizard;
    }

    protected void setWizard(ArchiveWizard wizard) {
      wizardRef = new WeakReference(wizard);
    }


}
