
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive.wizard;

//
import org.enhydra.tool.archive.ArchiveException;
import org.enhydra.tool.archive.JarPlan;

//
import java.awt.*;
import java.io.File;
import javax.swing.*;
import java.beans.*;
import java.lang.ref.WeakReference;
import javax.swing.border.*;

//
public class E3FilePanel extends ArchivePanel {
    private FilePanel     filePanel = null;
    private GridBagLayout layoutMain = null;
    private JPanel        panelApp = null;
    private GridBagLayout layoutApp = null;
    private JTextArea             textNote = null;

    public E3FilePanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void readPlan(JarPlan plan) throws ArchiveException {
        filePanel.setArchivePath(plan.getArchivePath());
        filePanel.getDDTableModel().clear();
        filePanel.getDDTable().setVisible(false);
    }

    public void writePlan(JarPlan plan) throws ArchiveException {
        validatePlan();
        plan.setArchivePath(filePanel.getArchivePath());
    }

    public void validatePlan() throws ArchiveException {
        filePanel.validatePlan();
    }

    public String getPageTitle() {
        return "Enhydra Application";
    }

    public String getInstructions() {
        StringBuffer buf = new StringBuffer();

        buf.append("Enter archive name for your Enhydra super servlet.");
        return buf.toString();
    }

    // override ArchivePanel
    protected void setWizardRef(ArchiveWizard wizard) {
      super.setWizard(wizard);
      filePanel.setWizard(wizard);
    }

    protected void preparePanel() {
        String[]     defaultDD = new String[0];
        StringBuffer buf = new StringBuffer();

        filePanel.setDefaultDD(defaultDD);
        filePanel.preparePanel();
    }

    private void pmInit() {
        filePanel.setExtension("jar");
    }

    private void jbInit() throws Exception {
        filePanel = (FilePanel) Beans.instantiate(getClass().getClassLoader(),
                                                  FilePanel.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        panelApp = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                              JPanel.class.getName());
        layoutApp =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        textNote = (JTextArea) Beans.instantiate(getClass().getClassLoader(),
                                                 JTextArea.class.getName());

        textNote.setWrapStyleWord(true);
        textNote.setLineWrap(true);
        textNote.setEnabled(false);
        textNote.setText("Super servlet archives do not support deployment descriptors. You "
                         + "will need to deploy a seperate configuration file with the super "
                         + "servlet archive.");
        textNote.setAutoscrolls(false);
        textNote.setDisabledTextColor(UIManager.getColor("Label.foreground"));
        textNote.setBackground(UIManager.getColor("Label.background"));
        textNote.setEditable(false);
        textNote.setFont(new java.awt.Font("SansSerif", 0, 12));
        panelApp.setLayout(layoutApp);
        panelApp.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(new Color(187, 218, 252), new Color(91, 107, 123)),
                                            "Deployment"));
        panelApp.add(textNote,
                     new GridBagConstraints(0, 0, 1, 3, 0.4, 0.4,
                                            GridBagConstraints.CENTER,
                                            GridBagConstraints.BOTH,
                                            new Insets(10, 10, 10, 10), 10,
                                            10));
        this.setLayout(layoutMain);
        this.add(filePanel,
                 new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(0, 0, 0, 0), 0, 0));
        this.add(panelApp,
                 new GridBagConstraints(0, 1, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(5, 5, 5, 5), 0, 0));
    }

}
