
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive.wizard;

//
// ToolBox imports
//
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.archive.WarPlan;
import org.enhydra.tool.archive.ArchiveException;
import org.enhydra.tool.archive.ArchiveTool;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.SwingUtil;

//
// Standard imports
//
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.ResourceBundle;
import java.util.jar.Manifest;
import java.util.jar.JarInputStream;

//
public class WarType extends ArchiveType {
    public final static String EXT = "war";

    //
    protected String getClassIncludeRoot() {
        return "WEB-INF/CLASSES/";
    }

    protected final String getSelectionName() {
        return "Web Application";
    }

    protected final String getExtension() {
        return EXT;
    }

    protected final String getType() {
        return getExtension();
    }

    protected final String getDescription() {
        StringBuffer buf = new StringBuffer();

        buf.append("Create a web application archive for ");
        buf.append("deploying serlvet classes, libraries, ");
        buf.append("Java Server Pages and static content.");
        return buf.toString();
    }

    protected void initPanels() throws ArchiveException {
        WarFilePanel   filePanel = new WarFilePanel();
        ContentPanel   contentPanel = new ContentPanel();
        ClassesPanel   classesPanel = new ClassesPanel();
        LibraryPanel   libPanel = new LibraryPanel();
        ArchivePanel[] newPanels = new ArchivePanel[4];

        newPanels[0] = filePanel;
        newPanels[1] = contentPanel;
        newPanels[2] = classesPanel;
        newPanels[3] = libPanel;
        setWizardPanels(newPanels);
    }

    protected void initPlan() throws ArchiveException {
        WarPlan    warPlan = null;
        PathHandle path = null;

        setPlan(new WarPlan());
        getWarPlan().setArchivePath(getDefaultArchivePath());
        path = PathHandle.createPathHandle(getWarPlan().getArchivePath());
        if (path.isFile()) {
            readArchive(getWarPlan().getArchivePath());
        }
    }

    protected boolean readArchive(Manifest manifest, String[] entries) {
        boolean    read = super.readArchive(manifest, entries);
        PathHandle root = null;

        if (read) {
            String value = null;

            try {
                value = manifest.getMainAttributes().getValue(CONTENT_ROOT);
                root = PathHandle.createPathHandle(value);
                if (root.isDirectory()) {
                    PathHandle cursor = null;
                    String[]   content = new String[0];
                    ArrayList  list = new ArrayList();

                    getWarPlan().setContentRoot(root.getPath());
                    for (int i = 0; i < entries.length; i++) {
                        if (entries[i].toUpperCase().startsWith("WEB-INF")) {

                            // ignore
                        } else if (entries[i].toUpperCase().startsWith("META-INF")) {

                            // ignore
                        } else {
                            cursor =
                                PathHandle.createPathHandle(root.getPath()
                                                            + File.separator
                                                            + entries[i]);
                            if (cursor.isFile()) {
                                list.add(cursor.getPath());
                            }
                        }
                    }
                    list.trimToSize();
                    content = new String[list.size()];
                    content = (String[]) list.toArray(content);
                    list.clear();
                    getWarPlan().setContentFiles(content);
                }
            } catch (ArchiveException e) {
                e.printStackTrace(System.err);
            }
        }
        return read;
    }

    private WarPlan getWarPlan() throws ArchiveException {
        return (WarPlan) getPlan();
    }

}
