
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.common;

// JDK
import java.io.File;
import javax.swing.filechooser.FileFilter;

/**
 * An implementation of FileFilter that filters out all files
 * except for those matching a set of given extensions.
 */
public class FilenameFilter extends FileFilter
    implements java.io.FileFilter {
    private String  includeName = new String();
    private String  descTitle = new String();
    private boolean directoryValid = true;
    private boolean excludeOnly = false;

    /**
     * Creates an extension file filter without an extensions set.
     */
    public FilenameFilter() {}

    /**
     * Return true if this file should be shown in a file chooser.
     */
    public boolean accept(File f) {
        boolean    acceptFile = false;
        PathHandle handle = null;

        if (f == null) {
            acceptFile = true;
        } else if (f.isDirectory()) {
            acceptFile = isDirectoryValid();
        } else {
            handle = PathHandle.createPathHandle(f);
            acceptFile = handle.endsWith(File.separator + includeName);
        }
        return acceptFile;
    }

    /**
     * Returns the human readable description of this filter.
     */
    public String getDescription() {
        StringBuffer desc = new StringBuffer();

        desc.append(getDescriptionTitle());
        desc.append(' ');
        desc.append('(');
        desc.append(includeName);
        desc.append(')');
        return desc.toString();
    }

    /**
     * Sets the portion of the description that does not
     * include extensions.
     */
    public void setDescriptionTitle(String d) {
        this.descTitle = d;
    }

    /**
     * Gets the portion of the description that does not
     * include extensions.
     */
    public String getDescriptionTitle() {
        return descTitle;
    }

    public void setDirectoryValid(boolean valid) {
        directoryValid = valid;
    }

    public boolean isDirectoryValid() {
        return directoryValid;
    }

    public void setIncludeName(String n) {
        includeName = n;
    }

    public String getIncludeName() {
        return includeName;
    }



}
